@@ -44,10 +44,10 @@
 
 "use strict";
 
-var KEYWORDS = "break case catch const continue debugger default delete do else finally for function if in instanceof let new return switch throw try typeof var void while with";
+var KEYWORDS = "break case catch class const continue debugger default delete do else extends finally for function if in instanceof new return switch throw try typeof var void while with";
 var KEYWORDS_ATOM = "false null true";
 var RESERVED_WORDS = [
-    "await abstract boolean byte char class double enum export extends final float goto implements import int interface long native package private protected public short static super synchronized this throws transient volatile yield",
+    "abstract async await boolean byte char double enum export final float goto implements import int interface let long native package private protected public short static super synchronized this throws transient volatile yield",
     KEYWORDS_ATOM,
     KEYWORDS,
 ].join(" ");
@@ -100,14 +100,19 @@ var OPERATORS = makePredicate([
     "/=",
     "*=",
     "%=",
+    "**=",
     ">>=",
     "<<=",
     ">>>=",
+    "&=",
     "|=",
     "^=",
-    "&=",
     "&&",
-    "||"
+    "||",
+    "??",
+    "&&=",
+    "||=",
+    "??=",
 ]);
 
 var NEWLINE_CHARS = "\n\r\u2028\u2029";
@@ -119,7 +124,7 @@ var PUNC_AFTER_EXPRESSION = PUNC_SEPARATORS + PUNC_CLOSERS;
 var PUNC_BEFORE_EXPRESSION = PUNC_OPENERS + PUNC_SEPARATORS;
 var PUNC_CHARS = PUNC_BEFORE_EXPRESSION + "`" + PUNC_CLOSERS;
 var WHITESPACE_CHARS = NEWLINE_CHARS + " \u00a0\t\f\u000b\u200b\u2000\u2001\u2002\u2003\u2004\u2005\u2006\u2007\u2008\u2009\u200a\u202f\u205f\u3000\uFEFF";
-var NON_IDENTIFIER_CHARS = makePredicate(characters("./'\"" + OPERATOR_CHARS + PUNC_CHARS + WHITESPACE_CHARS));
+var NON_IDENTIFIER_CHARS = makePredicate(characters("./'\"#" + OPERATOR_CHARS + PUNC_CHARS + WHITESPACE_CHARS));
 
 NEWLINE_CHARS = makePredicate(characters(NEWLINE_CHARS));
 OPERATOR_CHARS = makePredicate(characters(OPERATOR_CHARS));
@@ -159,10 +164,10 @@ function decode_escape_sequence(seq) {
       case "t": return "\t";
       case "u":
         var code;
-        if (seq.length == 5) {
-            code = seq.slice(1);
-        } else if (seq[1] == "{" && seq.slice(-1) == "}") {
+        if (seq[1] == "{" && seq.slice(-1) == "}") {
             code = seq.slice(2, -1);
+        } else if (seq.length == 5) {
+            code = seq.slice(1);
         } else {
             return;
         }
@@ -237,10 +242,10 @@ function tokenizer($TEXT, filename, html5_comments, shebang) {
         read_template   : with_eof_error("Unterminated template literal", function(strings) {
             var s = "";
             for (;;) {
-                var ch = next(true, true);
+                var ch = read();
                 switch (ch) {
                   case "\\":
-                    ch += next(true, true);
+                    ch += read();
                     break;
                   case "`":
                     strings.push(s);
@@ -255,6 +260,11 @@ function tokenizer($TEXT, filename, html5_comments, shebang) {
                 }
                 s += ch;
             }
+
+            function read() {
+                var ch = next(true, true);
+                return ch == "\r" ? "\n" : ch;
+            }
         }),
     };
     var prev_was_dot = false;
@@ -346,7 +356,7 @@ function tokenizer($TEXT, filename, html5_comments, shebang) {
 
     function read_while(pred) {
         var ret = "", ch;
-        while ((ch = peek()) && pred(ch)) ret += next();
+        while ((ch = peek()) && pred(ch, ret)) ret += next();
         return ret;
     }
 
@@ -354,24 +364,27 @@ function tokenizer($TEXT, filename, html5_comments, shebang) {
         js_error(err, filename, S.tokline, S.tokcol, S.tokpos);
     }
 
+    function is_octal(num) {
+        return /^0[0-7_]+$/.test(num);
+    }
+
     function read_num(prefix) {
         var has_e = false, after_e = false, has_x = false, has_dot = prefix == ".";
-        var num = read_while(function(ch) {
-            var code = ch.charCodeAt(0);
-            switch (code) {
-              case 120: case 88: // xX
+        var num = read_while(function(ch, str) {
+            switch (ch) {
+              case "x": case "X":
                 return has_x ? false : (has_x = true);
-              case 101: case 69: // eE
+              case "e": case "E":
                 return has_x ? true : has_e ? false : (has_e = after_e = true);
-              case 43: case 45: // +-
+              case "+": case "-":
                 return after_e;
-              case (after_e = false, 46): // .
-                return (!has_dot && !has_x && !has_e) ? (has_dot = true) : false;
+              case (after_e = false, "."):
+                return has_dot || has_e || has_x || is_octal(str) ? false : (has_dot = true);
             }
-            return is_digit(code) || /[_0-9a-fo]/i.test(ch);
+            return /[_0-9a-dfo]/i.test(ch);
         });
         if (prefix) num = prefix + num;
-        if (/^0[0-7_]+$/.test(num)) {
+        if (is_octal(num)) {
             if (next_token.has_directive("use strict")) parse_error("Legacy octal literals are not allowed in strict mode");
         } else {
             num = num.replace(has_x ? /([1-9a-f]|.0)_(?=[0-9a-f])/gi : /([1-9]|.0)_(?=[0-9])/gi, "$1");
@@ -460,7 +473,7 @@ function tokenizer($TEXT, filename, html5_comments, shebang) {
     });
 
     function read_name() {
-        var backslash = false, name = "", ch, escaped = false, hex;
+        var backslash = false, ch, escaped = false, name = peek() == "#" ? next() : "";
         while (ch = peek()) {
             if (!backslash) {
                 if (ch == "\\") escaped = backslash = true, next();
@@ -475,7 +488,7 @@ function tokenizer($TEXT, filename, html5_comments, shebang) {
             }
         }
         if (KEYWORDS[name] && escaped) {
-            hex = name.charCodeAt(0).toString(16).toUpperCase();
+            var hex = name.charCodeAt(0).toString(16).toUpperCase();
             name = "\\u" + "0000".substr(hex.length) + hex + name.slice(1);
         }
         return name;
@@ -610,7 +623,7 @@ function tokenizer($TEXT, filename, html5_comments, shebang) {
             if (PUNC_CHARS[ch]) return token("punc", next());
             if (looking_at("=>")) return token("punc", next() + next());
             if (OPERATOR_CHARS[ch]) return read_operator();
-            if (code == 92 || !NON_IDENTIFIER_CHARS[ch]) return read_word();
+            if (code == 35 || code == 92 || !NON_IDENTIFIER_CHARS[ch]) return read_word();
             break;
         }
         parse_error("Unexpected character '" + ch + "'");
@@ -651,7 +664,7 @@ var UNARY_PREFIX = makePredicate("typeof void delete -- ++ ! ~ - +");
 
 var UNARY_POSTFIX = makePredicate("-- ++");
 
-var ASSIGNMENT = makePredicate("= += -= /= *= %= >>= <<= >>>= |= ^= &=");
+var ASSIGNMENT = makePredicate("= += -= /= *= %= **= >>= <<= >>>= &= |= ^= &&= ||= ??=");
 
 var PRECEDENCE = function(a, ret) {
     for (var i = 0; i < a.length;) {
@@ -662,6 +675,7 @@ var PRECEDENCE = function(a, ret) {
     }
     return ret;
 }([
+    ["??"],
     ["||"],
     ["&&"],
     ["|"],
@@ -844,6 +858,24 @@ function parse($TEXT, options) {
               case "await":
                 if (S.in_async) return simple_statement();
                 break;
+              case "export":
+                next();
+                return export_();
+              case "import":
+                var token = peek();
+                if (!(token.type == "punc" && /^[(.]$/.test(token.value))) {
+                    next();
+                    return import_();
+                }
+                break;
+              case "let":
+                if (is_vardefs()) {
+                    next();
+                    var node = let_();
+                    semicolon();
+                    return node;
+                }
+                break;
               case "yield":
                 if (S.in_generator) return simple_statement();
                 break;
@@ -878,6 +910,10 @@ function parse($TEXT, options) {
                 next();
                 return break_cont(AST_Break);
 
+              case "class":
+                next();
+                return class_(AST_DefClass);
+
               case "const":
                 next();
                 var node = const_();
@@ -925,12 +961,6 @@ function parse($TEXT, options) {
                 next();
                 return if_();
 
-              case "let":
-                next();
-                var node = let_();
-                semicolon();
-                return node;
-
               case "return":
                 if (S.in_function == 0 && !options.bare_returns)
                     croak("'return' outside of function");
@@ -1039,14 +1069,138 @@ function parse($TEXT, options) {
         return stat;
     }
 
+    function has_modifier(name) {
+        if (!is("name", name)) return;
+        var token = peek();
+        if (!token) return;
+        if (is_token(token, "operator", "=")) return;
+        if (token.type == "punc" && /^[(;}]$/.test(token.value)) return;
+        if (has_newline_before(token)) return;
+        return next();
+    }
+
+    function class_(ctor) {
+        var was_async = S.in_async;
+        var was_gen = S.in_generator;
+        S.input.push_directives_stack();
+        S.input.add_directive("use strict");
+        var name;
+        if (ctor === AST_DefClass) {
+            name = as_symbol(AST_SymbolDefClass);
+        } else {
+            name = as_symbol(AST_SymbolClass, true);
+        }
+        var parent = null;
+        if (is("keyword", "extends")) {
+            next();
+            handle_regexp();
+            parent = expr_atom(true);
+        }
+        expect("{");
+        var props = [];
+        while (!is("punc", "}")) {
+            if (is("punc", ";")) {
+                next();
+                continue;
+            }
+            var start = S.token;
+            var fixed = !!has_modifier("static");
+            var async = has_modifier("async");
+            if (is("operator", "*")) {
+                next();
+                var internal = is("name") && /^#/.test(S.token.value);
+                var key = as_property_key();
+                var gen_start = S.token;
+                var gen = function_(async ? AST_AsyncGeneratorFunction : AST_GeneratorFunction);
+                gen.start = gen_start;
+                gen.end = prev();
+                props.push(new AST_ClassMethod({
+                    start: start,
+                    static: fixed,
+                    private: internal,
+                    key: key,
+                    value: gen,
+                    end: prev(),
+                }));
+                continue;
+            }
+            var internal = is("name") && /^#/.test(S.token.value);
+            var key = as_property_key();
+            if (is("punc", "(")) {
+                var func_start = S.token;
+                var func = function_(async ? AST_AsyncFunction : AST_Function);
+                func.start = func_start;
+                func.end = prev();
+                props.push(new AST_ClassMethod({
+                    start: start,
+                    static: fixed,
+                    private: internal,
+                    key: key,
+                    value: func,
+                    end: prev(),
+                }));
+                continue;
+            }
+            if (async) unexpected(async);
+            var value = null;
+            if (is("operator", "=")) {
+                next();
+                S.in_async = false;
+                S.in_generator = false;
+                value = maybe_assign();
+                S.in_generator = was_gen;
+                S.in_async = was_async;
+            } else if (!(is("punc", ";") || is("punc", "}"))) {
+                var type = null;
+                switch (key) {
+                  case "get":
+                    type = AST_ClassGetter;
+                    break;
+                  case "set":
+                    type = AST_ClassSetter;
+                    break;
+                }
+                if (type) {
+                    props.push(new type({
+                        start: start,
+                        static: fixed,
+                        private: is("name") && /^#/.test(S.token.value),
+                        key: as_property_key(),
+                        value: create_accessor(),
+                        end: prev(),
+                    }));
+                    continue;
+                }
+            }
+            semicolon();
+            props.push(new AST_ClassField({
+                start: start,
+                static: fixed,
+                private: internal,
+                key: key,
+                value: value,
+                end: prev(),
+            }));
+        }
+        next();
+        S.input.pop_directives_stack();
+        S.in_generator = was_gen;
+        S.in_async = was_async;
+        return new ctor({
+            extends: parent,
+            name: name,
+            properties: props,
+        });
+    }
+
     function for_() {
         var await = is("name", "await") && next();
         expect("(");
         var init = null;
-        if (!is("punc", ";")) {
+        if (await || !is("punc", ";")) {
             init = is("keyword", "const")
                 ? (next(), const_(true))
-                : is("keyword", "let")
+                : is("name", "let") && is_vardefs()
                 ? (next(), let_(true))
                 : is("keyword", "var")
                 ? (next(), var_(true))
@@ -1094,6 +1248,7 @@ function parse($TEXT, options) {
     }
 
     function for_enum(ctor, init) {
+        handle_regexp();
         var obj = expression();
         expect(")");
         return new ctor({
@@ -1161,6 +1316,11 @@ function parse($TEXT, options) {
         }
         if (node instanceof AST_SymbolFunarg) return node;
         if (node instanceof AST_SymbolRef) return new AST_SymbolFunarg(node);
+        if (node instanceof AST_Yield) return new AST_SymbolFunarg({
+            start: node.start,
+            name: "yield",
+            end: node.end,
+        });
         token_error(node.start, "Invalid arrow parameter");
     }
 
@@ -1187,20 +1347,19 @@ function parse($TEXT, options) {
         if (is("punc", "{")) {
             body = block_();
             value = null;
-            if (S.input.has_directive("use strict")) {
-                argnames.forEach(strict_verify_symbol);
-            }
         } else {
             body = [];
+            handle_regexp();
             value = maybe_assign();
         }
+        var is_strict = S.input.has_directive("use strict");
         S.input.pop_directives_stack();
         --S.in_function;
         S.in_loop = loop;
         S.labels = labels;
         S.in_generator = was_gen;
         S.in_async = was_async;
-        return new (async ? AST_AsyncArrow : AST_Arrow)({
+        var node = new (async ? AST_AsyncArrow : AST_Arrow)({
             start: start,
             argnames: argnames,
             rest: rest,
@@ -1208,6 +1367,8 @@ function parse($TEXT, options) {
             value: value,
             end: prev(),
         });
+        if (is_strict) node.each_argname(strict_verify_symbol);
+        return node;
     }
 
     var function_ = function(ctor) {
@@ -1240,23 +1401,24 @@ function parse($TEXT, options) {
         S.in_loop = 0;
         S.labels = [];
         var body = block_();
-        if (S.input.has_directive("use strict")) {
-            if (name) strict_verify_symbol(name);
-            argnames.forEach(strict_verify_symbol);
-            if (argnames.rest) strict_verify_symbol(argnames.rest);
-        }
+        var is_strict = S.input.has_directive("use strict");
         S.input.pop_directives_stack();
         --S.in_function;
         S.in_loop = loop;
         S.labels = labels;
         S.in_generator = was_gen;
         S.in_async = was_async;
-        return new ctor({
+        var node = new ctor({
             name: name,
             argnames: argnames,
             rest: argnames.rest || null,
             body: body
         });
+        if (is_strict) {
+            if (name) strict_verify_symbol(name);
+            node.each_argname(strict_verify_symbol);
+        }
+        return node;
     };
 
     function if_() {
@@ -1272,6 +1434,199 @@ function parse($TEXT, options) {
         });
     }
 
+    function is_alias() {
+        return is("name") || is_identifier_string(S.token.value);
+    }
+
+    function export_() {
+        if (is("operator", "*")) {
+            next();
+            var alias = "*";
+            if (is("name", "as")) {
+                next();
+                if (!is_alias()) expect_token("name");
+                alias = S.token.value;
+                next();
+            }
+            expect_token("name", "from");
+            var path = S.token;
+            expect_token("string");
+            semicolon();
+            return new AST_ExportForeign({
+                aliases: [ alias ],
+                keys: [ "*" ],
+                path: path.value,
+                quote: path.quote,
+            });
+        }
+        if (is("punc", "{")) {
+            next();
+            var aliases = [];
+            var keys = [];
+            while (is_alias()) {
+                var key = S.token;
+                next();
+                keys.push(key);
+                if (is("name", "as")) {
+                    next();
+                    if (!is_alias()) expect_token("name");
+                    aliases.push(S.token.value);
+                    next();
+                } else {
+                    aliases.push(key.value);
+                }
+                if (!is("punc", "}")) expect(",");
+            }
+            expect("}");
+            if (is("name", "from")) {
+                next();
+                var path = S.token;
+                expect_token("string");
+                semicolon();
+                return new AST_ExportForeign({
+                    aliases: aliases,
+                    keys: keys.map(function(token) {
+                        return token.value;
+                    }),
+                    path: path.value,
+                    quote: path.quote,
+                });
+            }
+            semicolon();
+            return new AST_ExportReferences({
+                properties: keys.map(function(token, index) {
+                    if (!is_token(token, "name")) token_error(token, "Name expected");
+                    var sym = _make_symbol(AST_SymbolExport, token);
+                    sym.alias = aliases[index];
+                    return sym;
+                }),
+            });
+        }
+        if (is("keyword", "default")) {
+            next();
+            var start = S.token;
+            var body = export_default_decl();
+            if (body) {
+                body.start = start;
+                body.end = prev();
+            } else {
+                handle_regexp();
+                body = expression();
+                semicolon();
+            }
+            return new AST_ExportDefault({ body: body });
+        }
+        return new AST_ExportDeclaration({ body: export_decl() });
+    }
+
+    function maybe_named(def, expr) {
+        if (expr.name) {
+            expr = new def(expr);
+            expr.name = new (def === AST_DefClass ? AST_SymbolDefClass : AST_SymbolDefun)(expr.name);
+        }
+        return expr;
+    }
+
+    function export_default_decl() {
+        if (is("name", "async")) {
+            if (!is_token(peek(), "keyword", "function")) return;
+            next();
+            next();
+            if (!is("operator", "*")) return maybe_named(AST_AsyncDefun, function_(AST_AsyncFunction));
+            next();
+            return maybe_named(AST_AsyncGeneratorDefun, function_(AST_AsyncGeneratorFunction));
+        } else if (is("keyword")) switch (S.token.value) {
+          case "class":
+            next();
+            return maybe_named(AST_DefClass, class_(AST_ClassExpression));
+          case "function":
+            next();
+            if (!is("operator", "*")) return maybe_named(AST_Defun, function_(AST_Function));
+            next();
+            return maybe_named(AST_GeneratorDefun, function_(AST_GeneratorFunction));
+        }
+    }
+
+    var export_decl = embed_tokens(function() {
+        if (is("name")) switch (S.token.value) {
+          case "async":
+            next();
+            expect_token("keyword", "function");
+            if (!is("operator", "*")) return function_(AST_AsyncDefun);
+            next();
+            return function_(AST_AsyncGeneratorDefun);
+          case "let":
+            next();
+            var node = let_();
+            semicolon();
+            return node;
+        } else if (is("keyword")) switch (S.token.value) {
+          case "class":
+            next();
+            return class_(AST_DefClass);
+          case "const":
+            next();
+            var node = const_();
+            semicolon();
+            return node;
+          case "function":
+            next();
+            if (!is("operator", "*")) return function_(AST_Defun);
+            next();
+            return function_(AST_GeneratorDefun);
+          case "var":
+            next();
+            var node = var_();
+            semicolon();
+            return node;
+        }
+        unexpected();
+    });
+
+    function import_() {
+        var all = null;
+        var def = as_symbol(AST_SymbolImport, true);
+        var props = null;
+        if (def ? (def.key = "", is("punc", ",") && next()) : !is("string")) {
+            if (is("operator", "*")) {
+                next();
+                expect_token("name", "as");
+                all = as_symbol(AST_SymbolImport);
+                all.key = "*";
+            } else {
+                expect("{");
+                props = [];
+                while (is_alias()) {
+                    var alias;
+                    if (is_token(peek(), "name", "as")) {
+                        var key = S.token.value;
+                        next();
+                        next();
+                        alias = as_symbol(AST_SymbolImport);
+                        alias.key = key;
+                    } else {
+                        alias = as_symbol(AST_SymbolImport);
+                        alias.key = alias.name;
+                    }
+                    props.push(alias);
+                    if (!is("punc", "}")) expect(",");
+                }
+                expect("}");
+            }
+        }
+        if (all || def || props) expect_token("name", "from");
+        var path = S.token;
+        expect_token("string");
+        semicolon();
+        return new AST_Import({
+            all: all,
+            default: def,
+            path: path.value,
+            properties: props,
+            quote: path.quote,
+        });
+    }
+
     function block_() {
         expect("{");
         var a = [];
@@ -1379,6 +1734,11 @@ function parse($TEXT, options) {
         return a;
     }
 
+    function is_vardefs() {
+        var token = peek();
+        return is_token(token, "name") || is_token(token, "punc", "[") || is_token(token, "punc", "{");
+    }
+
     var const_ = function(no_in) {
         return new AST_Const({
             start       : prev(),
@@ -1406,58 +1766,66 @@ function parse($TEXT, options) {
     var new_ = function(allow_calls) {
         var start = S.token;
         expect_token("operator", "new");
-        var newexp = expr_atom(false), args;
-        if (is("punc", "(")) {
+        var call;
+        if (is("punc", ".") && is_token(peek(), "name", "target")) {
             next();
-            args = expr_list(")", !options.strict);
+            next();
+            call = new AST_NewTarget();
         } else {
-            args = [];
+            var exp = expr_atom(false), args;
+            if (is("punc", "(")) {
+                next();
+                args = expr_list(")", !options.strict);
+            } else {
+                args = [];
+            }
+            call = new AST_New({ expression: exp, args: args });
         }
-        var call = new AST_New({
-            start      : start,
-            expression : newexp,
-            args       : args,
-            end        : prev()
-        });
-        mark_pure(call);
+        call.start = start;
+        call.end = prev();
         return subscripts(call, allow_calls);
     };
 
     function as_atom_node() {
-        var tok = S.token, ret;
+        var ret, tok = S.token, value = tok.value;
         switch (tok.type) {
           case "num":
-            ret = new AST_Number({ start: tok, end: tok, value: tok.value });
-            break;
-          case "bigint":
-            ret = new AST_BigInt({ start: tok, end: tok, value: tok.value });
-            break;
-          case "string":
-            ret = new AST_String({
-                start : tok,
-                end   : tok,
-                value : tok.value,
-                quote : tok.quote
-            });
-            break;
-          case "regexp":
-            ret = new AST_RegExp({ start: tok, end: tok, value: tok.value });
-            break;
-          case "atom":
-            switch (tok.value) {
-              case "false":
-                ret = new AST_False({ start: tok, end: tok });
-                break;
-              case "true":
-                ret = new AST_True({ start: tok, end: tok });
-                break;
-              case "null":
-                ret = new AST_Null({ start: tok, end: tok });
-                break;
+            if (isFinite(value)) {
+                ret = new AST_Number({ value: value });
+            } else {
+                ret = new AST_Infinity();
+                if (value < 0) ret = new AST_UnaryPrefix({ operator: "-", expression: ret });
             }
             break;
+          case "bigint":
+            ret = new AST_BigInt({ value: value });
+            break;
+          case "string":
+            ret = new AST_String({ value : value, quote : tok.quote });
+            break;
+          case "regexp":
+            ret = new AST_RegExp({ value: value });
+            break;
+          case "atom":
+            switch (value) {
+              case "false":
+                ret = new AST_False();
+                break;
+              case "true":
+                ret = new AST_True();
+                break;
+              case "null":
+                ret = new AST_Null();
+                break;
+              default:
+                unexpected();
+            }
+            break;
+          default:
+            unexpected();
         }
         next();
+        ret.start = ret.end = tok;
         return ret;
     }
 
@@ -1469,10 +1837,7 @@ function parse($TEXT, options) {
         if (is("punc")) {
             switch (start.value) {
               case "`":
-                var tmpl = template(null);
-                tmpl.start = start;
-                tmpl.end = prev();
-                return subscripts(tmpl, allow_calls);
+                return subscripts(template(null), allow_calls);
               case "(":
                 next();
                 if (is("punc", ")")) {
@@ -1497,11 +1862,13 @@ function parse($TEXT, options) {
                 expect(")");
                 var end = prev();
                 end.comments_before = ex.end.comments_before;
-                [].push.apply(ex.end.comments_after, end.comments_after);
+                end.comments_after.forEach(function(comment) {
+                    ex.end.comments_after.push(comment);
+                    if (comment.nlb) S.token.nlb = true;
+                });
                 end.comments_after.length = 0;
                 end.comments_after = ex.end.comments_after;
                 ex.end = end;
-                if (ex instanceof AST_Call) mark_pure(ex);
                 if (is("punc", "=>")) return arrow(ex instanceof AST_Sequence ? ex.expressions : [ ex ], start);
                 return subscripts(ex, allow_calls);
               case "[":
@@ -1511,7 +1878,14 @@ function parse($TEXT, options) {
             }
             unexpected();
         }
-        if (is("keyword", "function")) {
+        if (is("keyword")) switch (start.value) {
+          case "class":
+            next();
+            var clazz = class_(AST_ClassExpression);
+            clazz.start = start;
+            clazz.end = prev();
+            return subscripts(clazz, allow_calls);
+          case "function":
             next();
             var func;
             if (is("operator", "*")) {
@@ -1618,7 +1992,7 @@ function parse($TEXT, options) {
                 var gen = function_(AST_GeneratorFunction);
                 gen.start = gen_start;
                 gen.end = prev();
-                a.push(new AST_ObjectKeyVal({
+                a.push(new AST_ObjectMethod({
                     start: start,
                     key: key,
                     value: gen,
@@ -1667,7 +2041,7 @@ function parse($TEXT, options) {
                 var func = function_(AST_Function);
                 func.start = func_start;
                 func.end = prev();
-                a.push(new AST_ObjectKeyVal({
+                a.push(new AST_ObjectMethod({
                     start: start,
                     key: key,
                     value: func,
@@ -1693,7 +2067,7 @@ function parse($TEXT, options) {
                 var func = function_(is_gen ? AST_AsyncGeneratorFunction : AST_AsyncFunction);
                 func.start = func_start;
                 func.end = prev();
-                a.push(new AST_ObjectKeyVal({
+                a.push(new AST_ObjectMethod({
                     start: start,
                     key: key,
                     value: func,
@@ -1753,9 +2127,21 @@ function parse($TEXT, options) {
 
     function _make_symbol(type, token) {
         var name = token.value;
-        if (name === "await" && S.in_async) unexpected(token);
-        if (name === "yield" && S.in_generator) unexpected(token);
-        return new (name === "this" ? AST_This : type)({
+        switch (name) {
+          case "await":
+            if (S.in_async) unexpected(token);
+            break;
+          case "super":
+            type = AST_Super;
+            break;
+          case "this":
+            type = AST_This;
+            break;
+          case "yield":
+            if (S.in_generator) unexpected(token);
+            break;
+        }
+        return new type({
             name: "" + name,
             start: token,
             end: token,
@@ -1763,7 +2149,7 @@ function parse($TEXT, options) {
     }
 
     function strict_verify_symbol(sym) {
-        if (sym.name == "arguments" || sym.name == "eval")
+        if (sym.name == "arguments" || sym.name == "eval" || sym.name == "let")
             token_error(sym.start, "Unexpected " + sym.name + " in strict mode");
     }
 
@@ -1859,20 +2245,8 @@ function parse($TEXT, options) {
         });
     }
 
-    function mark_pure(call) {
-        var start = call.start;
-        var comments = start.comments_before;
-        var i = HOP(start, "comments_before_length") ? start.comments_before_length : comments.length;
-        while (--i >= 0) {
-            var comment = comments[i];
-            if (/[@#]__PURE__/.test(comment.value)) {
-                call.pure = comment;
-                break;
-            }
-        }
-    }
-
     function template(tag) {
+        var start = tag ? tag.start : S.token;
         var read = S.input.context().read_template;
         var strings = [];
         var expressions = [];
@@ -1883,53 +2257,73 @@ function parse($TEXT, options) {
         }
         next();
         return new AST_Template({
+            start: start,
             expressions: expressions,
             strings: strings,
             tag: tag,
+            end: prev(),
         });
     }
 
-    var subscripts = function(expr, allow_calls) {
+    function subscripts(expr, allow_calls) {
         var start = expr.start;
-        if (is("punc", ".")) {
-            next();
-            return subscripts(new AST_Dot({
-                start      : start,
-                expression : expr,
-                property   : as_name(),
-                end        : prev()
-            }), allow_calls);
+        var optional = null;
+        while (true) {
+            if (is("operator", "?") && is_token(peek(), "punc", ".")) {
+                next();
+                next();
+                optional = expr;
+            }
+            if (is("punc", "[")) {
+                next();
+                var prop = expression();
+                expect("]");
+                expr = new AST_Sub({
+                    start: start,
+                    optional: optional === expr,
+                    expression: expr,
+                    property: prop,
+                    end: prev(),
+                });
+            } else if (allow_calls && is("punc", "(")) {
+                next();
+                expr = new AST_Call({
+                    start: start,
+                    optional: optional === expr,
+                    expression: expr,
+                    args: expr_list(")", !options.strict),
+                    end: prev(),
+                });
+            } else if (optional === expr || is("punc", ".")) {
+                if (optional !== expr) next();
+                expr = new AST_Dot({
+                    start: start,
+                    optional: optional === expr,
+                    expression: expr,
+                    property: as_name(),
+                    end: prev(),
+                });
+            } else if (is("punc", "`")) {
+                if (optional) croak("Invalid template on optional chain");
+                expr = template(expr);
+            } else {
+                break;
+            }
         }
-        if (is("punc", "[")) {
-            next();
-            var prop = expression();
-            expect("]");
-            return subscripts(new AST_Sub({
-                start      : start,
-                expression : expr,
-                property   : prop,
-                end        : prev()
-            }), allow_calls);
-        }
-        if (allow_calls && is("punc", "(")) {
-            next();
-            var call = new AST_Call({
-                start      : start,
-                expression : expr,
-                args       : expr_list(")", !options.strict),
-                end        : prev()
-            });
-            mark_pure(call);
-            return subscripts(call, true);
-        }
-        if (is("punc", "`")) {
-            var tmpl = template(expr);
-            tmpl.start = expr.start;
-            tmpl.end = prev();
-            return subscripts(tmpl, allow_calls);
+        if (optional) expr.terminal = true;
+        if (expr instanceof AST_Call && !expr.pure) {
+            var start = expr.start;
+            var comments = start.comments_before;
+            var i = HOP(start, "comments_before_length") ? start.comments_before_length : comments.length;
+            while (--i >= 0) {
+                if (/[@#]__PURE__/.test(comments[i].value)) {
+                    expr.pure = true;
+                    break;
+                }
+            }
         }
         return expr;
-    };
+    }
 
     function maybe_unary(no_in) {
         var start = S.token;
@@ -2038,7 +2432,7 @@ function parse($TEXT, options) {
     };
 
     function is_assignable(expr) {
-        return expr instanceof AST_PropAccess || expr instanceof AST_SymbolRef;
+        return expr instanceof AST_PropAccess && !expr.optional || expr instanceof AST_SymbolRef;
     }
 
     function to_destructured(node) {
@@ -2160,7 +2554,7 @@ function parse($TEXT, options) {
         while (!is("eof"))
             body.push(statement());
         S.input.pop_directives_stack();
-        var end = prev();
+        var end = prev() || start;
         var toplevel = options.toplevel;
         if (toplevel) {
             toplevel.body = toplevel.body.concat(body);
