@@ -72,7 +72,7 @@ fields: {
             console.log(k, o[k]);
         console.log(o.q);
     }
-    expect_exact: 'var o=new class A{"#p";static #p="PASS";async;get q(){return A.#p}[6*7]=console?"foo":"bar"};for(var k in o)console.log(k,o[k]);console.log(o.q);'
+    expect_exact: 'var o=new class A{"#p";static#p="PASS";async;get q(){return A.#p}[6*7]=console?"foo":"bar"};for(var k in o)console.log(k,o[k]);console.log(o.q);'
     expect_stdout: [
         "42 foo",
         "#p undefined",
@@ -136,7 +136,7 @@ private_methods: {
             }
         }().q.then(console.log);
     }
-    expect_exact: "(new class A{static*#f(){yield 3*A.#p}async #g(){for(var a of A.#f())return a*await 2}static get #p(){return 7}get q(){return this.#g()}}).q.then(console.log);"
+    expect_exact: "(new class A{static*#f(){yield 3*A.#p}async#g(){for(var a of A.#f())return a*await 2}static get#p(){return 7}get q(){return this.#g()}}).q.then(console.log);"
     expect_stdout: "42"
     node_version: ">=14.6"
 }
@@ -260,6 +260,9 @@ block_scoped: {
     expect: {
         "use strict";
         0;
+        {
+            class A {}
+        }
         if (console) {
             class B {}
         }
@@ -269,6 +272,38 @@ block_scoped: {
     node_version: ">=4"
 }
 
+retain_declaration: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        "use strict";
+        var a = "FAIL";
+        try {
+            console.log(function() {
+                return a;
+                class a {}
+            }());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        var a = "FAIL";
+        try {
+            console.log(function() {
+                return a;
+                class a {}
+            }());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 drop_extends: {
     options = {
         inline: true,
@@ -399,6 +434,33 @@ static_side_effects: {
         console.log(a);
     }
     expect: {
+        var a = "FAIL 1";
+        (class {
+            static c = a = "PASS";
+        });
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+static_side_effects_strict: {
+    options = {
+        inline: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        var a = "FAIL 1";
+        class A {
+            static p = a = "PASS";
+            q = a = "FAIL 2";
+        }
+        console.log(a);
+    }
+    expect: {
+        "use strict";
         var a = "FAIL 1";
         a = "PASS";
         console.log(a);
@@ -632,6 +694,7 @@ collapse_non_strict: {
 collapse_rhs: {
     options = {
         collapse_vars: true,
+        unsafe: true,
     }
     input: {
         "use strict";
@@ -771,13 +834,40 @@ unused_await: {
         })();
     }
     expect: {
+        var await = "PASS";
+        (async function() {
+            (class {
+                static c = console.log(await);
+            });
+        })();
+    }
+    expect_stdout: true
+    node_version: ">=12 <16"
+}
+
+unused_await_strict: {
+    options = {
+        inline: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        var await = "PASS";
+        (async function() {
+            class A {
+                static p = console.log(await);
+            }
+        })();
+    }
+    expect: {
+        "use strict";
         var await = "PASS";
         (async function() {
             (() => console.log(await))();
         })();
     }
     expect_stdout: true
-    node_version: ">=12"
+    node_version: ">=12 <16"
 }
 
 computed_key_side_effects: {
@@ -854,7 +944,7 @@ keep_fnames: {
 issue_805_1: {
     options = {
         inline: true,
-        passes: 2,
+        passes: 3,
         pure_getters: "strict",
         reduce_vars: true,
         sequences: true,
@@ -890,7 +980,7 @@ issue_805_1: {
 issue_805_2: {
     options = {
         inline: true,
-        passes: 2,
+        passes: 3,
         pure_getters: "strict",
         reduce_vars: true,
         sequences: true,
@@ -937,9 +1027,9 @@ issue_4681: {
     }
     expect: {
         console.log(function(a) {
-            class A {
+            (class {
                 static p = a = this;
-            }
+            });
             return typeof a;
         }());
     }
@@ -1113,6 +1203,31 @@ issue_4705: {
         }
     }
     expect: {
+        (class {
+            [console.log("PASS")]() {}
+        });
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_4705_strict: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        var a = "PASS";
+        class A {
+            p = a = "FAIL";
+            [console.log(a)];
+        }
+    }
+    expect: {
+        "use strict";
         console.log("PASS");
     }
     expect_stdout: "PASS"
@@ -1121,7 +1236,7 @@ issue_4705: {
 
 issue_4720: {
     options = {
-        ie8: true,
+        ie: true,
         reduce_vars: true,
         toplevel: true,
         unused: true,
@@ -1274,6 +1389,7 @@ issue_4725_1: {
 
 issue_4725_2: {
     options = {
+        if_return: true,
         inline: true,
     }
     input: {
@@ -1334,9 +1450,40 @@ issue_4756: {
     expect: {
         try {
             (class extends 42 {
-                [console.log("foo")]() {}
-            }),
-            (() => console.log("bar"))();
+                static [console.log("foo")] = console.log("bar");
+            });
+        } catch (e) {
+            console.log("baz");
+        }
+    }
+    expect_stdout: [
+        "foo",
+        "baz",
+    ]
+    node_version: ">=12"
+}
+
+issue_4756_strict: {
+    options = {
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        try {
+            class A extends 42 {
+                static [console.log("foo")] = console.log("bar");
+            }
+        } catch (e) {
+            console.log("baz");
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            (class extends 42 {
+                static [console.log("foo")] = console.log("bar");
+            });
         } catch (e) {
             console.log("baz");
         }
@@ -1363,9 +1510,9 @@ issue_4821_1: {
     }
     expect: {
         var a;
-        class A {
+        (class {
             static p = void (a = this);
-        }
+        });
         console.log(typeof a);
     }
     expect_stdout: "function"
@@ -1403,7 +1550,7 @@ issue_4829_1: {
     input: {
         "use strict";
         try {
-            class A extends { f(){} }.f {}
+            class A extends { f() {} }.f {}
         } catch (e) {
             console.log("PASS");
         }
@@ -1541,3 +1688,791 @@ issue_4848: {
     expect_stdout: "PASS"
     node_version: ">=4"
 }
+
+drop_unused_self_reference: {
+    options = {
+        pure_getters: "strict",
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        class A {}
+        (A.p = A).q = console.log("PASS");
+    }
+    expect: {
+        "use strict";
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_4951_1: {
+    input: {
+        class A {
+            static#p = console.log("PASS");
+        }
+    }
+    expect_exact: 'class A{static#p=console.log("PASS")}'
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_4951_2: {
+    input: {
+        new class {
+            constructor() {
+                this.#f().then(console.log);
+            }
+            async#f() {
+                return await "PASS";
+            }
+        }();
+    }
+    expect_exact: 'new class{constructor(){this.#f().then(console.log)}async#f(){return await"PASS"}};'
+    expect_stdout: "PASS"
+    node_version: ">=14.6"
+}
+
+issue_4962_1: {
+    options = {
+        ie: true,
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            function f() {
+                while (console.log(typeof g));
+            }
+            class A {
+                static p = f();
+            }
+        })(function g() {});
+    }
+    expect: {
+        (function g() {}),
+        void class {
+            static c = function() {
+                while (console.log(typeof g));
+            }();
+        };
+    }
+    expect_stdout: "undefined"
+    node_version: ">=12"
+}
+
+issue_4962_1_strict: {
+    options = {
+        ie: true,
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        (function() {
+            function f() {
+                while (console.log(typeof g));
+            }
+            class A {
+                static p = f();
+            }
+        })(function g() {});
+    }
+    expect: {
+        "use strict";
+        (function g() {});
+        while (console.log(typeof g));
+    }
+    expect_stdout: "undefined"
+    node_version: ">=12"
+}
+
+issue_4962_2: {
+    options = {
+        ie: true,
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        console.log(function f() {}(function g() {
+            function h() {
+                f;
+            }
+            class A {
+                static p = h();
+            }
+        }, typeof g));
+    }
+    expect: {
+        console.log(function f() {}(function g() {
+            (class {
+                static c = f;
+            });
+        }));
+    }
+    expect_stdout: "undefined"
+    node_version: ">=12"
+}
+
+issue_4962_2_strict: {
+    options = {
+        ie: true,
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        console.log(function f() {}(function g() {
+            function h() {
+                f;
+            }
+            class A {
+                static p = h();
+            }
+        }, typeof g));
+    }
+    expect: {
+        "use strict";
+        console.log(function f() {}(function g() {
+            f;
+        }));
+    }
+    expect_stdout: "undefined"
+    node_version: ">=12"
+}
+
+issue_4982_1: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        "use strict";
+        try {} catch (e) {
+            class A extends 42 {}
+        }
+        console.log("PASS");
+    }
+    expect: {
+        "use strict";
+        {
+            class A {}
+        }
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_4982_2: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        var a = "PASS";
+        try {} catch (e) {
+            class A {
+                static p = a = "FAIL";
+            }
+        }
+        console.log(a);
+    }
+    expect: {
+        var a = "PASS";
+        {
+            class A {}
+        }
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_4992: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        class A {
+            static P = this;
+            get p() {}
+        }
+        console.log(typeof A.P);
+    }
+    expect: {
+        console.log(typeof class {
+            static P = this;
+            get p() {}
+        }.P);
+    }
+    expect_stdout: "function"
+    node_version: ">=12"
+}
+
+issue_4996_1: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = 1;
+        console.log(new class A {
+            p = a-- && new A();
+        }().p.p);
+    }
+    expect: {
+        var a = 1;
+        console.log(new class A {
+            p = a-- && new A();
+        }().p.p);
+    }
+    expect_stdout: "0"
+    node_version: ">=12"
+}
+
+issue_4996_2: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 1;
+        console.log(new class A {
+            p = a-- && new A();
+        }().p.p);
+    }
+    expect: {
+        var a = 1;
+        console.log(new class A {
+            p = a-- && new A();
+        }().p.p);
+    }
+    expect_stdout: "0"
+    node_version: ">=12"
+}
+
+issue_5015_1: {
+    options = {
+        side_effects: true,
+    }
+    input: {
+        "use strict";
+        var a;
+        try {
+            (class a {
+                [a]() {}
+            });
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        var a;
+        try {
+            (class a {
+                [a]() {}
+            });
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5015_2: {
+    options = {
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        try {
+            new class A {
+                [(A, 42)]() {}
+            }();
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            new class A {
+                [(A, 42)]() {}
+            }();
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5015_3: {
+    options = {
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        (class A {
+            static f() {
+                return A;
+            }
+        });
+        console.log("PASS");
+    }
+    expect: {
+        "use strict";
+        (class A {});
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5015_4: {
+    options = {
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        (class A {
+            static f() {
+                return A;
+            }
+        });
+        console.log("PASS");
+    }
+    expect: {
+        "use strict";
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5053_1: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        try {
+            console.log(new class A {
+                constructor() {
+                    A = 42;
+                }
+            }());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            console.log(new class A {
+                constructor() {
+                    A = 42;
+                }
+            }());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5053_2: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        try {
+            console.log(new class A {
+                f() {
+                    A = 42;
+                }
+            }().f());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            console.log(new class A {
+                f() {
+                    A = 42;
+                }
+            }().f());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5053_3: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        try {
+            console.log(new class A {
+                p = A = 42;
+            }().p);
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        try {
+            console.log(new class A {
+                p = A = 42;
+            }().p);
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5053_4: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        class A {
+            constructor() {
+                A = 42;
+            }
+        }
+        try {
+            console.log(new A());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        class A {
+            constructor() {
+                A = 42;
+            }
+        }
+        try {
+            console.log(new A());
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5082_1: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            class A {
+                p = console.log("PASS");
+                q() {}
+            }
+            class B {
+                static P = new A();
+            }
+        })();
+    }
+    expect: {
+        (function() {
+            class A {
+                p = console.log("PASS");
+                q() {}
+            }
+            (class {
+                static c = new A();
+            });
+        })();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5082_1_strict: {
+    options = {
+        inline: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        (function() {
+            class A {
+                p = console.log("PASS");
+                q() {}
+            }
+            class B {
+                static P = new A();
+            }
+        })();
+    }
+    expect: {
+        "use strict";
+        (function() {
+            class A {
+                p = console.log("PASS");
+                q() {}
+            }
+            new A();
+        })();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5082_2: {
+    options = {
+        inline: true,
+        passes: 2,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        (function() {
+            class A {
+                p = console.log("PASS");
+                q() {}
+            }
+            class B {
+                static P = new A();
+            }
+        })();
+    }
+    expect: {
+        (function() {
+            class A {
+                p = console.log("PASS");
+                q() {}
+            }
+            (class {
+                static c = new A();
+            });
+        })();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5082_2_static: {
+    options = {
+        inline: true,
+        passes: 2,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        (function() {
+            class A {
+                p = console.log("PASS");
+                q() {}
+            }
+            class B {
+                static P = new A();
+            }
+        })();
+    }
+    expect: {
+        "use strict";
+        void new class {
+            p = console.log("PASS");
+            q() {}
+        }();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5142: {
+    options = {
+        evaluate: true,
+        merge_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = 0, b;
+        if (++a)
+            new class {
+                p = b = null;
+                constructor(c) {
+                    console.log(c ? "FAIL" : "PASS");
+                }
+            }(b, a);
+    }
+    expect: {
+        var a = 0, b;
+        if (++a)
+            new class {
+                p = b = null;
+                constructor(c) {
+                    console.log(c ? "FAIL" : "PASS");
+                }
+            }(b, 1);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5294_1: {
+    options = {
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        (class A {
+            static p = console.log(typeof A);
+        });
+    }
+    expect: {
+        (class A {
+            static c = console.log(typeof A);
+        });
+    }
+    expect_stdout: "function"
+    node_version: ">=12"
+}
+
+issue_5294_2: {
+    options = {
+        reduce_vars: true,
+        side_effects: true,
+        toplevel: true,
+    }
+    input: {
+        class A {
+            static p = console.log(typeof A);
+        }
+    }
+    expect: {
+        class A {
+            static p = console.log(typeof A);
+        }
+    }
+    expect_stdout: "function"
+    node_version: ">=12"
+}
+
+issue_5294_3: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = this;
+        (class A {
+            static p = console.log(a === A ? "FAIL" : "PASS");
+        });
+    }
+    expect: {
+        var a = this;
+        (class A {
+            static p = console.log(a === A ? "FAIL" : "PASS");
+        });
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5294_4: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        (class A {
+            static p = function() {
+                var a = this;
+                console.log(a === A ? "FAIL" : "PASS");
+            }();
+        });
+    }
+    expect: {
+        (class A {
+            static p = function() {
+                console.log(this === A ? "FAIL" : "PASS");
+            }();
+        });
+    }
+    expect_stdout: "PASS"
+    node_version: ">=12"
+}
+
+issue_5322: {
+    options = {
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 41;
+        class A {
+            static p() {
+                console.log(++a);
+            }
+            static q = this.p();
+        }
+    }
+    expect: {
+        var a = 41;
+        (class {
+            static p() {
+                console.log(++a);
+            }
+            static q = this.p();
+        });
+    }
+    expect_stdout: "42"
+    node_version: ">=12"
+}
