@@ -939,6 +939,185 @@ catch_return_assign: {
     expect_stdout: "PASS"
 }
 
+catch_return_assign_may_throw: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f() {
+            try {
+                throw "FAIL";
+            } catch (e) {
+                return e = console.log("PASS");
+            }
+        }
+        f();
+    }
+    expect: {
+        function f() {
+            try {
+                throw "FAIL";
+            } catch (e) {
+                return console.log("PASS");
+            }
+        }
+        f();
+    }
+    expect_stdout: "PASS"
+}
+
+finally_return_assign: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        console.log(function(a) {
+            try {
+                throw "FAIL";
+            } finally {
+                return a = "PASS";
+            }
+        }());
+    }
+    expect: {
+        console.log(function(a) {
+            try {
+                throw "FAIL";
+            } finally {
+                return "PASS";
+            }
+        }());
+    }
+    expect_stdout: "PASS"
+}
+
+last_assign_statement: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f(a) {
+            a = a("PASS");
+        }
+        f(console.log);
+    }
+    expect: {
+        function f(a) {
+            a("PASS");
+        }
+        f(console.log);
+    }
+    expect_stdout: "PASS"
+}
+
+last_assign_if_else: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f(a) {
+            if (a)
+                a = console.log("foo");
+            else {
+                console.log("bar");
+                a = console.log("baz");
+            }
+        }
+        f(42);
+        f(null);
+    }
+    expect: {
+        function f(a) {
+            if (a)
+                console.log("foo");
+            else {
+                console.log("bar");
+                console.log("baz");
+            }
+        }
+        f(42);
+        f(null);
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "baz",
+    ]
+}
+
+last_assign_catch: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f() {
+            try {
+                throw "FAIL";
+            } catch (e) {
+                e = console.log("PASS");
+            }
+        }
+        f();
+    }
+    expect: {
+        function f() {
+            try {
+                throw "FAIL";
+            } catch (e) {
+                console.log("PASS");
+            }
+        }
+        f();
+    }
+    expect_stdout: "PASS"
+}
+
+last_assign_finally: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        function f(a) {
+            try {
+                throw a.log;
+            } catch (e) {
+                a = e;
+            } finally {
+                a = a("PASS");
+            }
+        }
+        f(console);
+    }
+    expect: {
+        function f(a) {
+            try {
+                throw a.log;
+            } catch (e) {
+                a = e;
+            } finally {
+                a("PASS");
+            }
+        }
+        f(console);
+    }
+    expect_stdout: "PASS"
+}
+
+consecutive_assignments: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        while (a = void 0, a = "PASS", console.log(a));
+        var a;
+    }
+    expect: {
+        while (void 0, a = "PASS", console.log(a));
+        var a;
+    }
+    expect_stdout: "PASS"
+}
+
 issue_3578: {
     options = {
         dead_code: true,
@@ -1420,3 +1599,73 @@ issue_4570: {
     }
     expect_stdout: "NaN"
 }
+
+issue_5030: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        (function(a, b) {
+            a = function f() {
+                if (a)
+                    if (b--)
+                        setImmediate(f);
+                    else
+                        console.log("FAIL");
+                else
+                    console.log("PASS");
+            }();
+        })(42, 1);
+    }
+    expect: {
+        (function(a, b) {
+            a = function f() {
+                if (a)
+                    if (b--)
+                        setImmediate(f);
+                    else
+                        console.log("FAIL");
+                else
+                    console.log("PASS");
+            }();
+        })(42, 1);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=0.12"
+}
+
+issue_5106_1: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        console.log(typeof function(a) {
+            return a = arguments;
+        }("FAIL")[0]);
+    }
+    expect: {
+        console.log(typeof function(a) {
+            return a = arguments;
+        }("FAIL")[0]);
+    }
+    expect_stdout: "object"
+}
+
+issue_5106_2: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        "use strict";
+        console.log(function(a) {
+            return a = arguments;
+        }("PASS")[0]);
+    }
+    expect: {
+        "use strict";
+        console.log(function(a) {
+            return arguments;
+        }("PASS")[0]);
+    }
+    expect_stdout: "PASS"
+}
