@@ -70,7 +70,7 @@ object_shorthand_assign: {
         ({ a = "PASS" } = 42);
         console.log(a);
     }
-    expect_exact: '({a:a="PASS"}=42);console.log(a);'
+    expect_exact: '({a="PASS"}=42);console.log(a);'
     expect_stdout: "PASS"
     node_version: ">=6"
 }
@@ -80,7 +80,7 @@ object_shorthand_declaration: {
         var { a = "PASS" } = 42;
         console.log(a);
     }
-    expect_exact: 'var{a:a="PASS"}=42;console.log(a);'
+    expect_exact: 'var{a="PASS"}=42;console.log(a);'
     expect_stdout: "PASS"
     node_version: ">=6"
 }
@@ -91,7 +91,7 @@ object_shorthand_function: {
             console.log(a);
         })(42);
     }
-    expect_exact: '(function({a:a="PASS"}){console.log(a)})(42);'
+    expect_exact: '(function({a="PASS"}){console.log(a)})(42);'
     expect_stdout: "PASS"
     node_version: ">=6"
 }
@@ -149,7 +149,7 @@ process_boolean_returns: {
     }
     expect: {
         console.log(function(a = console.log("FAIL 1")) {
-            return a() ? "PASS" : "FAIL 2";
+            return 42 ? "PASS" : "FAIL 2";
         }(function() {
             return 1;
         }));
@@ -158,6 +158,28 @@ process_boolean_returns: {
     node_version: ">=6"
 }
 
+collapse_arg_sequence: {
+    options = {
+        collapse_vars: true,
+        unused: true,
+    }
+    input: {
+        (function(a = (console.log("bar"), console.log)) {
+            a("foo");
+        })();
+    }
+    expect: {
+        (function(a = console.log("bar")) {
+            (0, console.log)("foo");
+        })();
+    }
+    expect_stdout: [
+        "bar",
+        "foo",
+    ]
+    node_version: ">=6"
+}
+
 collapse_value_1: {
     options = {
         collapse_vars: true,
@@ -245,21 +267,64 @@ maintain_if: {
     node_version: ">=6"
 }
 
-reduce_value: {
+reduce_funarg: {
     options = {
         evaluate: true,
+        keep_fargs: false,
         reduce_vars: true,
         unused: true,
     }
     input: {
-        console.log(function(a = "PASS") {
-            return a;
-        }());
+        console.log(...function(a = "foo", b = "bar", c = "baz") {
+            return [ a, b, c ];
+        }(void 0, null));
     }
     expect: {
-        console.log("PASS");
+        console.log(...function() {
+            return [ "foo", null, "baz" ];
+        }());
     }
-    expect_stdout: "PASS"
+    expect_stdout: "foo null baz"
+    node_version: ">=6"
+}
+
+reduce_array: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unsafe: true,
+        unused: true,
+    }
+    input: {
+        var [ a = "foo", b = "bar", c = "baz" ] = [ void 0, null ];
+        console.log(a, b, c);
+    }
+    expect: {
+        var [ c = "baz" ] = [];
+        console.log("foo", null, c);
+    }
+    expect_stdout: "foo null baz"
+    node_version: ">=6"
+}
+
+reduce_object: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unsafe: true,
+        unused: true,
+    }
+    input: {
+        var { a = "foo", b = "bar", c = "baz" } = { a: void 0, b: null };
+        console.log(a, b, c);
+    }
+    expect: {
+        var { c = "baz" } = {};
+        console.log("foo", null, c);
+    }
+    expect_stdout: "foo null baz"
     node_version: ">=6"
 }
 
@@ -422,7 +487,9 @@ inline_loop_1: {
 inline_loop_2: {
     options = {
         inline: true,
+        sequences: true,
         toplevel: true,
+        unused: true,
     }
     input: {
         while (function(a = [ "PASS" ]) {
@@ -432,10 +499,11 @@ inline_loop_2: {
         }());
     }
     expect: {
-        while (a = [ "PASS" ], a = function f(b) {
-            console.log(a[b]);
-        }(0), void 0) ;
-        var a;
+        while (a = [ "PASS" ],
+            b = void 0,
+            b = 0,
+            void (a = void console.log(a[b])));
+        var a, b;
     }
     expect_stdout: "PASS"
     node_version: ">=6"
@@ -522,6 +590,20 @@ retain_empty_iife: {
     node_version: ">=6"
 }
 
+drop_new_function: {
+    options = {
+        side_effects: true,
+    }
+    input: {
+        new function(a = console.log("PASS")) {}();
+    }
+    expect: {
+        void console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 retain_fargs: {
     options = {
         unused: true,
@@ -598,6 +680,7 @@ unused_var_1: {
 
 unused_var_2: {
     options = {
+        pure_getters: "strict",
         toplevel: true,
         unused: true,
     }
@@ -609,11 +692,7 @@ unused_var_2: {
         };
     }
     expect: {
-        var {
-            p: [] = [ console.log("FAIL") ],
-        } = {
-            p: [ console.log("PASS") ],
-        };
+        console.log("PASS");
     }
     expect_stdout: "PASS"
     node_version: ">=6"
@@ -676,7 +755,7 @@ unused_value_var_2: {
         console.log(a);
     }
     expect: {
-        var [ a ] = [ "PASS" ];
+        var a = [ "PASS" ][0];
         console.log(a);
     }
     expect_stdout: "PASS"
@@ -980,7 +1059,7 @@ mangle_arrow_1: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=6"
+    node_version: ">=6.9.3"
 }
 
 mangle_arrow_1_toplevel: {
@@ -1016,7 +1095,7 @@ mangle_arrow_1_toplevel: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=6"
+    node_version: ">=6.9.3"
 }
 
 mangle_arrow_2: {
@@ -1052,7 +1131,7 @@ mangle_arrow_2: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=6"
+    node_version: ">=6.9.3"
 }
 
 mangle_arrow_2_toplevel: {
@@ -1088,7 +1167,7 @@ mangle_arrow_2_toplevel: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=6"
+    node_version: ">=6.9.3"
 }
 
 issue_4444: {
@@ -1257,7 +1336,7 @@ issue_4468: {
     expect: {
         (function() {
             var {
-                [console.log("PASS")]: b = 0,
+                [console.log("PASS")]: b,
             } = 0;
         })();
     }
@@ -1416,6 +1495,7 @@ issue_4502_1: {
     expect: {
         (function() {
             var a = "PASS";
+            void 0,
             console.log(a),
             a++,
             void 0;
@@ -1439,6 +1519,7 @@ issue_4502_2: {
     expect: {
         (function() {
             var a = "PASS";
+            void 0,
             console.log(a),
             a++,
             void 0;
@@ -1528,7 +1609,7 @@ issue_4510_2: {
         };
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 issue_4523: {
@@ -1664,7 +1745,7 @@ issue_4588_2_evaluate: {
 
 issue_4817: {
     options = {
-        ie8: true,
+        ie: true,
         inline: true,
         unused: true,
     }
@@ -1681,3 +1762,450 @@ issue_4817: {
     expect_stdout: "function"
     node_version: ">=6"
 }
+
+issue_4854: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        console.log(function(a) {
+            (function(b = a = "foo") {
+                [] = "foo";
+            })();
+            a;
+        }());
+    }
+    expect: {
+        console.log(void 0);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
+
+issue_4916: {
+    options = {
+        collapse_vars: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+    }
+    input: {
+        var log = console.log;
+        (function(b = "foo") {
+            b.value = "FAIL";
+            b;
+            log(b.value);
+        })();
+    }
+    expect: {
+        var log = console.log;
+        (function(b = "foo") {
+            b.value = "FAIL";
+            b;
+            log(b.value);
+        })();
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
+
+issue_4994: {
+    options = {
+        loops: true,
+        unused: true,
+    }
+    input: {
+        var a = "FAIL";
+        (function(b = function() {
+            for (a in { PASS: 42 });
+        }()) {
+            var a;
+        })();
+        console.log(a);
+    }
+    expect: {
+        var a = "FAIL";
+        (function(b = function() {
+            for (a in { PASS: 42 });
+        }()) {})();
+        console.log(a);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5057_1: {
+    options = {
+        collapse_vars: true,
+        inline: true,
+        sequences: true,
+        unused: true,
+    }
+    input: {
+        var a = 42;
+        (function() {
+            var b = function(c = (console.log("foo"), b = a)) {
+                a && console.log("bar");
+            }();
+        })();
+    }
+    expect: {
+        var a = 42;
+        console.log("foo"),
+        void (a && console.log("bar"));
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=6"
+}
+
+issue_5057_2: {
+    options = {
+        inline: true,
+        unused: true,
+    }
+    input: {
+        (function f(a) {
+            (function(b = console.log("FAIL")) {})(a);
+        })(42);
+        console.log(typeof b);
+    }
+    expect: {
+        (function(a) {
+            [ b = console.log("FAIL") ] = [ a ],
+            void 0;
+            var b;
+        })(42);
+        console.log(typeof b);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
+
+issue_5057_3: {
+    options = {
+        inline: 3,
+        unused: true,
+    }
+    input: {
+        (function(a) {
+            (function f(b) {
+                (function(a = console.log("FAIL 1")) {})(b);
+                console.log(a);
+            })("FAIL 2");
+        })("PASS");
+    }
+    expect: {
+        (function(a) {
+            (function(b) {
+                (function(a = console.log("FAIL 1")) {})(b);
+                console.log(a);
+            })("FAIL 2");
+        })("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5057_4: {
+    options = {
+        if_return: true,
+        inline: true,
+        unused: true,
+    }
+    input: {
+        (function(a) {
+            (function f(b) {
+                (function(a = console.log("FAIL 1")) {})(b);
+                console.log(a);
+            })("FAIL 2");
+        })("PASS");
+    }
+    expect: {
+        (function(a) {
+            var b = "FAIL 2";
+            (function(a = console.log("FAIL 1")) {})(b);
+            console.log(a);
+        })("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5065: {
+    options = {
+        pure_getters: "strict",
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var [ a = console.log("PASS") ] = [ (A = 42).p ];
+    }
+    expect: {
+        var [ a = console.log("PASS") ] = [ (A = 42).p ];
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5138_1: {
+    options = {
+        evaluate: true,
+    }
+    input: {
+        console.log(function(a, b = a = "FAIL") {
+            return a;
+        }() && "PASS");
+    }
+    expect: {
+        console.log(function(a, b = a = "FAIL") {
+            return a;
+        }() && "PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5138_2: {
+    options = {
+        evaluate: true,
+    }
+    input: {
+        console.log(function(a, b = a = "FAIL 1") {
+            return a;
+        }(null, "FAIL 2") || "PASS");
+    }
+    expect: {
+        console.log((null, "PASS"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5192: {
+    options = {
+        dead_code: true,
+        ie: true,
+    }
+    input: {
+        (function a(a,  [] = a = "PASS") {
+            console.log(a);
+        })("FAIL");
+    }
+    expect: {
+        (function a(a,  [] = a = "PASS") {
+            console.log(a);
+        })("FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5246_1: {
+    options = {
+        keep_fargs: false,
+        pure_getters: true,
+        unused: true,
+    }
+    input: {
+        console.log(function({} = 42) {
+            return "PASS";
+        }("foo"));
+    }
+    expect: {
+        console.log(function() {
+            return "PASS";
+        }());
+    }
+    expect_stdout: "PASS"
+    expect_warnings: [
+        "INFO: Dropping unused default argument {}=42 [test/compress/default-values.js:1,29]",
+    ]
+    node_version: ">=6"
+}
+
+issue_5246_2: {
+    options = {
+        keep_fargs: false,
+        unused: true,
+    }
+    input: {
+        (function f(a = "FAIL", [] = 42) {
+            console.log(a);
+        })("PASS", []);
+    }
+    expect: {
+        (function(a = "FAIL") {
+            console.log(a);
+        })("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5246_3: {
+    options = {
+        default_values: true,
+        keep_fargs: false,
+        unused: true,
+    }
+    input: {
+        console.log(function f([ , {} ] = null){}([ , {} ]));
+    }
+    expect: {
+        console.log(function([ {} ]){}([ {} ]));
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
+
+issue_5256: {
+    options = {
+        inline: true,
+    }
+    input: {
+        (function(arguments = console.log) {
+            console;
+        })();
+        console.log(typeof arguments);
+    }
+    expect: {
+        // Syntax error on Node.js v6
+        (function(arguments = console.log) {
+            console;
+        })();
+        console.log(typeof arguments);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=8"
+}
+
+issue_5314_1: {
+    options = {
+        side_effects: true,
+    }
+    input: {
+        A = this;
+        new function() {
+            (function(a = console.log(this === A ? "PASS" : "FAIL")) {})();
+        }();
+    }
+    expect: {
+        A = this;
+        (function() {
+            (function(a = console.log(this === A ? "PASS" : "FAIL")) {})();
+        })();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5314_2: {
+    options = {
+        side_effects: true,
+    }
+    input: {
+        A = this;
+        new function() {
+            ((a = console.log(this === A ? "FAIL" : "PASS")) => {})();
+        }();
+    }
+    expect: {
+        A = this;
+        new function() {
+            console.log(this === A ? "FAIL" : "PASS");
+        }();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5336: {
+    options = {
+        default_values: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        do {
+            (function f(b = console.log("PASS")) {
+                a = f;
+            })(42);
+        } while (a());
+    }
+    expect: {
+        var a;
+        do {
+            (function f(b = console.log("PASS")) {
+                a = f;
+            })(42);
+        } while (a());
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5340_1: {
+    options = {
+        keep_fargs: true,
+        pure_getters: "strict",
+        unused: true,
+    }
+    input: {
+        var a;
+        (function(b = 42) {})(({ p: a } = true).q);
+        console.log(a);
+    }
+    expect: {
+        var a;
+        (function(b = 0) {})(({ p: a } = true).q);
+        console.log(a);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
+
+issue_5340_2: {
+    options = {
+        keep_fargs: true,
+        pure_getters: "strict",
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        (function(b = 42) {})(({ p: a } = true).q);
+        console.log(a);
+    }
+    expect: {
+        var a;
+        [ [].e = 0 ] = [ ({ p: a } = true).q ];
+        console.log(a);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
+
+issue_5340_3: {
+    options = {
+        keep_fargs: false,
+        pure_getters: "strict",
+        unused: true,
+    }
+    input: {
+        var a;
+        (function(b = 42) {})(({ p: a } = true).q);
+        console.log(a);
+    }
+    expect: {
+        var a;
+        (function() {})(a = true["p"]);
+        console.log(a);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=6"
+}
