@@ -795,9 +795,9 @@ issue_1656: {
         beautify: true,
     }
     input: {
-        for(var a=0;;);
+        for (var a=0;;);
     }
-    expect_exact: "for (;;) ;"
+    expect_exact: "for (;;);"
 }
 
 issue_1709: {
@@ -1110,7 +1110,7 @@ issue_1838: {
     }
     expect_exact: [
         "function f() {",
-        "    for (a; c; ) ;",
+        "    for (a; c; );",
         "}",
     ]
 }
@@ -1233,8 +1233,11 @@ issue_2105_2: {
 issue_2105_3: {
     options = {
         inline: true,
-        passes: 2,
+        join_vars: true,
+        passes: 3,
         reduce_vars: true,
+        side_effects: true,
+        sequences: true,
         unused: true,
     }
     input: {
@@ -1258,12 +1261,12 @@ issue_2105_3: {
         });
     }
     expect: {
-        !void void {
+        ({
             prop: function() {
-                console.log;
-                void console.log("PASS");
-            }
-        }.prop();
+                console.log,
+                console.log("PASS");
+            },
+        }).prop();
     }
     expect_stdout: "PASS"
 }
@@ -1557,9 +1560,9 @@ issue_2665: {
     }
     expect: {
         var a = 1;
-        !function g() {
+        (function g() {
             a-- && g();
-        }();
+        })();
         console.log(a);
     }
     expect_stdout: "-1"
@@ -1728,7 +1731,8 @@ issue_2768: {
     }
     expect: {
         var a = "FAIL";
-        var c = (d = a, void (d && (a = "PASS")));
+        d = a;
+        var c = void (d && (a = "PASS"));
         var d;
         console.log(a, typeof c);
     }
@@ -1765,7 +1769,7 @@ issue_2846: {
 issue_805_1: {
     options = {
         inline: true,
-        passes: 2,
+        passes: 3,
         pure_getters: "strict",
         reduce_vars: true,
         sequences: true,
@@ -1798,7 +1802,7 @@ issue_805_1: {
 issue_805_2: {
     options = {
         inline: true,
-        passes: 2,
+        passes: 3,
         pure_getters: "strict",
         reduce_vars: true,
         sequences: true,
@@ -2345,7 +2349,7 @@ function_argument_reference: {
 
 function_parameter_ie8: {
     options = {
-        ie8: true,
+        ie: true,
         reduce_vars: true,
         unused: true,
     }
@@ -2371,6 +2375,7 @@ function_parameter_ie8: {
 issue_3664: {
     options = {
         pure_getters: "strict",
+        side_effects: true,
         unused: true,
     }
     input: {
@@ -2381,7 +2386,8 @@ issue_3664: {
     }
     expect: {
         console.log(function() {
-            var b = ([ b && console.log("FAIL") ].p = 0, 0);
+            a = (a = [ b && console.log("FAIL") ]).p = 0;
+            var a, b = 0;
             return "PASS";
         }());
     }
@@ -2391,6 +2397,7 @@ issue_3664: {
 issue_3673: {
     options = {
         pure_getters: "strict",
+        sequences: true,
         side_effects: true,
         toplevel: true,
         unused: true,
@@ -2401,8 +2408,6 @@ issue_3673: {
         console.log("PASS");
     }
     expect: {
-        var a;
-        (a = [ a ]).p = 42;
         console.log("PASS");
     }
     expect_stdout: "PASS"
@@ -2551,10 +2556,9 @@ issue_3899: {
         console.log(typeof a);
     }
     expect: {
-        function a() {
+        console.log(typeof function() {
             return 2;
-        }
-        console.log(typeof a);
+        });
     }
     expect_stdout: "function"
 }
@@ -2652,7 +2656,7 @@ issue_3956: {
         collapse_vars: true,
         evaluate: true,
         inline: true,
-        passes: 2,
+        passes: 3,
         reduce_vars: true,
         sequences: true,
         side_effects: true,
@@ -2783,7 +2787,7 @@ issue_3986: {
     expect_stdout: "0"
 }
 
-issue_4017: {
+issue_4017_1: {
     options = {
         pure_getters: "strict",
         reduce_vars: true,
@@ -2801,7 +2805,31 @@ issue_4017: {
         var a = 0;
         console.log(function() {
             c &= 0;
-            var c;
+            var c = a++ + (A = a);
+        }());
+    }
+    expect_stdout: "undefined"
+}
+
+issue_4017_2: {
+    options = {
+        passes: 2,
+        pure_getters: "strict",
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        console.log(function f() {
+            var b = c &= 0;
+            var c = a++ + (A = a);
+            var d = c && c[f];
+        }());
+    }
+    expect: {
+        var a = 0;
+        console.log(function() {
+            0;
             a++,
             A = a;
         }());
@@ -2916,7 +2944,7 @@ issue_4133: {
         console.log(a);
     }
     expect: {
-        var b = 1;
+        var a = 1;
         console.log(0);
     }
     expect_stdout: "0"
@@ -3062,7 +3090,7 @@ issue_4184: {
     expect_stdout: "42"
 }
 
-issue_4235: {
+issue_4235_1: {
     options = {
         inline: true,
         reduce_vars: true,
@@ -3080,12 +3108,35 @@ issue_4235: {
         })();
     }
     expect: {
+        void function() {
+            var f = console.log(f);
+        }();
+    }
+    expect_stdout: "undefined"
+}
+
+issue_4235_2: {
+    options = {
+        inline: true,
+        passes: 2,
+        reduce_vars: true,
+        side_effects: true,
+        unused: true,
+        varify: true,
+    }
+    input: {
         (function() {
-            f = console.log(f),
-            void 0;
-            var f;
+            {
+                const f = 0;
+            }
+            (function f() {
+                var f = console.log(f);
+            })();
         })();
     }
+    expect: {
+        console.log(void 0);
+    }
     expect_stdout: "undefined"
 }
 
@@ -3221,7 +3272,7 @@ issue_4558_1: {
     expect: {
         var a = 0;
         var b = c >>>= a;
-        var c;
+        var c = 0;
         b && a++,
         console.log(a);
     }
@@ -3231,7 +3282,7 @@ issue_4558_1: {
 issue_4558_2: {
     options = {
         evaluate: true,
-        ie8: true,
+        ie: true,
         reduce_vars: true,
         unused: true,
     }
@@ -3380,7 +3431,7 @@ issue_4834: {
     }
     expect: {
         try {
-            void b.p;
+            b.p;
         } catch (e) {
             console.log("PASS");
         }
@@ -3388,3 +3439,178 @@ issue_4834: {
     }
     expect_stdout: "PASS"
 }
+
+issue_4912_1: {
+    options = {
+        pure_getters: "strict",
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = A = function() {};
+        A;
+        a.prototype = {
+            f: function() {
+                console.log("PASS");
+            },
+        };
+        new A().f();
+    }
+    expect: {
+        var a = A = function() {};
+        A;
+        a.prototype = {
+            f: function() {
+                console.log("PASS");
+            },
+        };
+        new A().f();
+    }
+    expect_stdout: "PASS"
+}
+
+issue_4912_2: {
+    options = {
+        pure_getters: "strict",
+        unused: true,
+    }
+    input: {
+        console.log(function() {
+            var g, f = function() {};
+            f.p = {};
+            (g = f.p.q = function() {}).r = "PASS";
+            return f;
+        }().p.q.r);
+    }
+    expect: {
+        console.log(function() {
+            var g, f = function() {};
+            f.p = {};
+            (f.p.q = function() {}).r = "PASS";
+            return f;
+        }().p.q.r);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_4912_3: {
+    options = {
+        pure_getters: "strict",
+        reduce_vars: true,
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        console.log(function(f, g) {
+            f = function() {};
+            f.p = {};
+            g = f.p.q = function() {};
+            g.r = "PASS";
+            return f;
+        }().p.q.r);
+    }
+    expect: {
+        console.log(function(f, g) {
+            f = function() {};
+            f.p = {};
+            g = f.p.q = function() {};
+            g.r = "PASS";
+            return f;
+        }().p.q.r);
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5079: {
+    options = {
+        collapse_vars: true,
+        pure_getters: "strict",
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        do {
+            (a = 123456).p = a;
+            a.q = null;
+        } while (console.log("PASS"));
+    }
+    expect: {
+        do {
+            0, 0, null;
+        } while (console.log("PASS"));
+    }
+    expect_stdout: "PASS"
+}
+
+issue_5224: {
+    options = {
+        evaluate: true,
+        keep_fargs: false,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function f() {
+            try {
+                var b = function() {
+                    var a = "FAIL 1";
+                    null && a;
+                    a = console.log(a);
+                }(new function(c, d) {
+                    console.log(d);
+                    a;
+                }("FAIL 2", Infinity));
+            } finally {
+                return f;
+            }
+        }
+        f();
+    }
+    expect: {
+        (function f() {
+            try {
+                (function() {
+                    var a = "FAIL 1";
+                    null;
+                    a = console.log(a);
+                })(function() {
+                    console.log(1 / 0);
+                    a;
+                }());
+            } finally {
+                return f;
+            }
+        })();
+    }
+    expect_stdout: "Infinity"
+}
+
+issue_5271: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        function f() {
+            do {
+                var a = b = 0 ^ f, b = b;
+            } while (console.log(42 - b));
+        }
+        f();
+    }
+    expect: {
+        (function f() {
+            do {
+                var b;
+                b = 0 ^ f;
+            } while (console.log(42 - b));
+        })();
+    }
+    expect_stdout: "42"
+}
