@@ -684,26 +684,47 @@ prototype_function: {
         side_effects: true,
     }
     input: {
-        var a = ({valueOf: 0}) < 1;
-        var b = ({toString: 0}) < 1;
-        var c = ({valueOf: 0}) + "";
-        var d = ({toString: 0}) + "";
-        var e = (({valueOf: 0}) + "")[2];
-        var f = (({toString: 0}) + "")[2];
-        var g = ({valueOf: 0}).valueOf();
-        var h = ({toString: 0}).toString();
+        function v() {
+            return this.valueOf === v ? "PASS" : "FAIL";
+        }
+        console.log(({ valueOf: v }) < 1);
+        console.log(({ valueOf: v }) + "");
+        console.log((( {valueOf: v }) + "")[2]);
+        console.log(({ valueOf: v }).valueOf());
+        function t() {
+            return this.toString === t ? "PASS" : "FAIL";
+        }
+        console.log(({ toString: t }) < 1);
+        console.log(({ toString: t }) + "");
+        console.log((( {toString: t }) + "")[2]);
+        console.log(({ toString: t }).toString());
     }
     expect: {
-        var a = ({valueOf: 0}) < 1;
-        var b = ({toString: 0}) < 1;
-        var c = ({valueOf: 0}) + "";
-        var d = ({toString: 0}) + "";
-        var e = (({valueOf: 0}) + "")[2];
-        var f = (({toString: 0}) + "")[2];
-        var g = 0();
-        var h = 0();
+        function v() {
+            return this.valueOf === v ? "PASS" : "FAIL";
+        }
+        console.log(({ valueOf: v }) < 1);
+        console.log(({ valueOf: v }) + "");
+        console.log((( {valueOf: v }) + "")[2]);
+        console.log(({ valueOf: v }).valueOf());
+        function t() {
+            return this.toString === t ? "PASS" : "FAIL";
+        }
+        console.log(({ toString: t }) < 1);
+        console.log(({ toString: t }) + "");
+        console.log((( {toString: t }) + "")[2]);
+        console.log(({ toString: t }).toString());
     }
-    expect_stdout: true
+    expect_stdout: [
+        "false",
+        "PASS",
+        "S",
+        "PASS",
+        "false",
+        "PASS",
+        "S",
+        "PASS",
+    ]
 }
 
 call_args: {
@@ -2731,8 +2752,7 @@ issue_3944: {
     }
     expect: {
         void function f() {
-            while (a = 0 == (a = void 0), console.log(a), void 0);
-            var a;
+            while (0 == void 0, console.log(false), void 0);
             f;
         }();
     }
@@ -3181,3 +3201,43 @@ issue_4552: {
     }
     expect_stdout: "NaN"
 }
+
+issue_4886_1: {
+    options = {
+        evaluate: true,
+        unsafe: true,
+    }
+    input: {
+        console.log("length" in {
+            __proto__: function() {},
+            length: void 0,
+        });
+    }
+    expect: {
+        console.log("length" in {
+            __proto__: function() {},
+            length: void 0,
+        });
+    }
+    expect_stdout: "true"
+}
+
+issue_4886_2: {
+    options = {
+        evaluate: true,
+        unsafe: true,
+    }
+    input: {
+        console.log("foo" in {
+            "foo": null,
+            __proto__: 42,
+        });
+    }
+    expect: {
+        console.log("foo" in {
+            "foo": null,
+            __proto__: 42,
+        });
+    }
+    expect_stdout: "true"
+}
