@@ -489,6 +489,160 @@ join_object_assignments_regex: {
     expect_stdout: "1"
 }
 
+chained_assignments: {
+    options = {
+        join_vars: true,
+    }
+    input: {
+        var a, b = a = {};
+        b.p = "PASS";
+        console.log(a.p);
+    }
+    expect: {
+        var a, b = a = {
+            p: "PASS",
+        };
+        console.log(a.p);
+    }
+    expect_stdout: "PASS"
+}
+
+folded_assignments_1: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+    }
+    input: {
+        var a = {};
+        a[a.PASS = 42] = "PASS";
+        console.log(a[42], a.PASS);
+    }
+    expect: {
+        var a = {
+            PASS: 42,
+            42: "PASS",
+        };
+        console.log(a[42], a.PASS);
+    }
+    expect_stdout: "PASS 42"
+}
+
+folded_assignments_2: {
+    options = {
+        evaluate: true,
+        join_vars: true,
+    }
+    input: {
+        "use strict";
+        var a = {};
+        a[42] = "FAIL";
+        a[a.PASS = 42] = "PASS";
+        console.log(a[42], a.PASS);
+    }
+    expect: {
+        "use strict";
+        var a = {
+            42: "FAIL",
+            PASS: 42,
+        };
+        a[42] = "PASS";
+        console.log(a[42], a.PASS);
+    }
+    expect_stdout: "PASS 42"
+}
+
+inlined_assignments: {
+    options = {
+        join_vars: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        (a = {}).p = "PASS";
+        console.log(a.p);
+    }
+    expect: {
+        var a = {
+            p: "PASS",
+        };
+        console.log(a.p);
+    }
+    expect_stdout: "PASS"
+}
+
+inilne_for: {
+    options = {
+        inline: true,
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = function() {
+            for (; console.log("PASS"););
+        };
+        a();
+    }
+    expect: {
+        for (; console.log("PASS"););
+    }
+    expect_stdout: "PASS"
+}
+
+inilne_var: {
+    options = {
+        inline: true,
+        join_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        A = "PASS";
+        var a = function() {
+            var b = A;
+            for (b in console.log(b));
+        };
+        a();
+    }
+    expect: {
+        A = "PASS";
+        var b = A;
+        for (b in console.log(b));
+    }
+    expect_stdout: "PASS"
+}
+
+typescript_enum: {
+    rename = true
+    options = {
+        assignments: true,
+        collapse_vars: true,
+        evaluate: true,
+        hoist_props: true,
+        inline: true,
+        join_vars: true,
+        passes: 4,
+        reduce_vars: true,
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var Enum;
+        (function (Enum) {
+            Enum[Enum.PASS = 42] = "PASS";
+        })(Enum || (Enum = {}));
+        console.log(Enum[42], Enum.PASS);
+    }
+    expect: {
+        console.log("PASS", 42);
+    }
+    expect_stdout: "PASS 42"
+}
+
 issue_2816: {
     options = {
         join_vars: true,
@@ -1015,7 +1169,7 @@ issue_3856: {
         console.log(function() {
             (function() {
                 var a, b;
-                if (a) return !!a;
+                if (a) return a, 1;
                 for (a = 0; !console;);
                 return 0;
             })();
@@ -1024,7 +1178,7 @@ issue_3856: {
     expect_stdout: "undefined"
 }
 
-issue_3916: {
+issue_3916_1: {
     options = {
         join_vars: true,
     }
@@ -1044,8 +1198,8 @@ issue_3916: {
         var o = {
             p: "PASS",
             __proto__: 42,
-            q: "FAIL",
         };
+        o.q = "FAIL";
         o.__proto__ = {
             p: "FAIL",
             q: "PASS",
@@ -1056,6 +1210,62 @@ issue_3916: {
     expect_stdout: "object PASS true PASS"
 }
 
+issue_3916_2: {
+    options = {
+        join_vars: true,
+    }
+    input: {
+        var log = console.log, o = {};
+        o.p = "FAIL 1";
+        o.__proto__ = {
+            get p() {
+                return "FAIL 2";
+            },
+            set p(u) {
+                log("FAIL 3");
+            },
+            set q(v) {
+                log("PASS 1");
+            },
+            get q() {
+                return "PASS 3";
+            },
+        };
+        o.p = "PASS 2";
+        o.q = "FAIL 4";
+        log(o.p);
+        log(o.q);
+    }
+    expect: {
+        var log = console.log, o = {
+            p: "FAIL 1",
+            __proto__: {
+                get p() {
+                    return "FAIL 2";
+                },
+                set p(u) {
+                    log("FAIL 3");
+                },
+                set q(v) {
+                    log("PASS 1");
+                },
+                get q() {
+                    return "PASS 3";
+                },
+            },
+        };
+        o.p = "PASS 2";
+        o.q = "FAIL 4";
+        log(o.p);
+        log(o.q);
+    }
+    expect_stdout: [
+        "PASS 1",
+        "PASS 2",
+        "PASS 3",
+    ]
+}
+
 assign_var: {
     options = {
         join_vars: true,
@@ -1116,10 +1326,7 @@ assign_sequence_var: {
         console.log(a, b, c);
     }
     expect: {
-        var a = 0, b = 1;
-        console.log(a),
-        a++;
-        var b = 2, c = 3;
+        var a = 0, b = 1, c = (console.log(a), a++, b = 2, 3);
         console.log(a, b, c);
     }
     expect_stdout: [
@@ -1127,3 +1334,26 @@ assign_sequence_var: {
         "1 2 3",
     ]
 }
+
+issue_5175: {
+    options = {
+        join_vars: true,
+    }
+    input: {
+        function log(f) {
+            console.log(f(), A.p);
+        }
+        log(function() {
+            return (A = {}).p = "PASS";
+        });
+    }
+    expect: {
+        function log(f) {
+            console.log(f(), A.p);
+        }
+        log(function() {
+            return (A = {}).p = "PASS";
+        });
+    }
+    expect_stdout: "PASS PASS"
+}
