@@ -1,4 +1,4 @@
-retain_block: {
+retain_block_1: {
     options = {}
     input: {
         "use strict";
@@ -20,6 +20,127 @@ retain_block: {
     node_version: ">=4"
 }
 
+retain_block_2: {
+    options = {
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        {
+            var a;
+            let a;
+        }
+    }
+    expect: {
+        "use strict";
+        {
+            var a;
+            let a;
+        }
+    }
+    expect_stdout: true
+    node_version: ">=4"
+}
+
+retain_block_2_mangle: {
+    rename = true
+    mangle = {
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        {
+            var a;
+            let a;
+        }
+    }
+    expect: {
+        "use strict";
+        {
+            var t;
+            let t;
+        }
+    }
+}
+
+retain_block_3: {
+    options = {
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        {
+            let a;
+            var a;
+        }
+    }
+    expect: {
+        "use strict";
+        {
+            let a;
+            var a;
+        }
+    }
+    expect_stdout: true
+    node_version: ">=4"
+}
+
+retain_block_3_mangle: {
+    rename = true
+    mangle = {
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        {
+            let a;
+            var a;
+        }
+    }
+    expect: {
+        "use strict";
+        {
+            let t;
+            var t;
+        }
+    }
+}
+
+retain_assignment: {
+    options = {
+        dead_code: true,
+        reduce_vars: true,
+    }
+    input: {
+        "use strict";
+        function f() {
+            return a = 0;
+            let a;
+        }
+        try {
+            f();
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        function f() {
+            return a = 0;
+            let a;
+        }
+        try {
+            f();
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 retain_catch: {
     options = {
         dead_code: true,
@@ -149,8 +270,8 @@ merge_vars_3: {
             var b = console;
             console.log(typeof b);
         }
-        var a = 1;
-        console.log(typeof a);
+        var b = 1;
+        console.log(typeof b);
     }
     expect_stdout: [
         "object",
@@ -159,6 +280,38 @@ merge_vars_3: {
     node_version: ">=4"
 }
 
+merge_vars_4: {
+    options = {
+        merge_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        var a = 1;
+        console.log(typeof a);
+        {
+            var b = console;
+            console.log(typeof b);
+            let a = 0;
+        }
+    }
+    expect: {
+        "use strict";
+        var a = 1;
+        console.log(typeof a);
+        {
+            var b = console;
+            console.log(typeof b);
+            let a = 0;
+        }
+    }
+    expect_stdout: [
+        "number",
+        "object",
+    ]
+    node_version: ">=4"
+}
+
 use_before_init_1: {
     options = {
         evaluate: true,
@@ -461,6 +614,41 @@ reduce_vars_3: {
     node_version: ">=4"
 }
 
+reduce_lambda: {
+    options = {
+        evaluate: true,
+        functions: true,
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        "use strict";
+        let f = function() {
+            console.log(a, b);
+        };
+        let a = "foo", b = 42;
+        f();
+        b = "bar";
+        f();
+    }
+    expect: {
+        "use strict";
+        function f() {
+            console.log("foo", b);
+        }
+        let b = 42;
+        f();
+        b = "bar";
+        f();
+    }
+    expect_stdout: [
+        "foo 42",
+        "foo bar",
+    ]
+    node_version: ">=4"
+}
+
 hoist_props: {
     options = {
         hoist_props: true,
@@ -536,6 +724,38 @@ loop_block_2: {
     node_version: ">=4"
 }
 
+do_break: {
+    options = {
+        loops: true,
+    }
+    input: {
+        "use strict";
+        try {
+            do {
+                if (a)
+                    break;
+                let a;
+            } while (!console);
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        try {
+            do {
+                if (a)
+                    break;
+                let a;
+            } while (!console);
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 do_continue: {
     options = {
         loops: true,
@@ -596,6 +816,82 @@ dead_block_after_return: {
     node_version: ">=4"
 }
 
+if_return_1: {
+    options = {
+        if_return: true,
+    }
+    input: {
+        "use strict";
+        function f(a) {
+            function g() {
+                return b = "PASS";
+            }
+            if (a)
+                return g();
+            let b;
+            return g();
+        };
+        console.log(f());
+    }
+    expect: {
+        "use strict";
+        function f(a) {
+            function g() {
+                return b = "PASS";
+            }
+            if (a)
+                return g();
+            let b;
+            return g();
+        };
+        console.log(f());
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+if_return_2: {
+    options = {
+        if_return: true,
+    }
+    input: {
+        "use strict";
+        function f(a) {
+            function g() {
+                return b = "FAIL";
+            }
+            if (a)
+                return g();
+            let b;
+            return g();
+        };
+        try {
+            console.log(f(42));
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect: {
+        "use strict";
+        function f(a) {
+            function g() {
+                return b = "FAIL";
+            }
+            if (a)
+                return g();
+            let b;
+            return g();
+        };
+        try {
+            console.log(f(42));
+        } catch (e) {
+            console.log("PASS");
+        }
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 do_if_continue_1: {
     options = {
         if_return: true,
@@ -790,12 +1086,10 @@ issue_4202: {
         "use strict";
         {
             let o = {};
-            (function() {
-                function f() {
-                    o.p = 42;
-                }
-                f(f);
-            })();
+            function f() {
+                o.p = 42;
+            }
+            f(f);
             console.log(o.p++);
         }
     }
@@ -897,6 +1191,7 @@ issue_4210: {
 issue_4212_1: {
     options = {
         dead_code: true,
+        reduce_vars: true,
     }
     input: {
         "use strict";
@@ -987,7 +1282,7 @@ issue_4225: {
 
 issue_4229: {
     options = {
-        ie8: true,
+        ie: true,
         side_effects: true,
     }
     input: {
@@ -1018,7 +1313,7 @@ issue_4229: {
 
 issue_4231: {
     options = {
-        ie8: true,
+        ie: true,
         side_effects: true,
     }
     input: {
@@ -1376,7 +1671,7 @@ issue_4438: {
                 ;
             else {
                 let a = console.log;
-                a("PASS");
+                void a("PASS");
             }
         }
         f();
@@ -1387,7 +1682,7 @@ issue_4438: {
 
 issue_4531_1: {
     mangle = {
-        ie8: true,
+        ie: true,
         toplevel: true,
     }
     input: {
@@ -1413,11 +1708,11 @@ issue_4531_1: {
 issue_4531_2: {
     options = {
         evaluate: true,
-        ie8: true,
+        ie: true,
         toplevel: true,
     }
     mangle = {
-        ie8: true,
+        ie: true,
         toplevel: true,
     }
     input: {
@@ -1546,3 +1841,185 @@ issue_4848: {
     expect_stdout: "PASS"
     node_version: ">=4"
 }
+
+issue_4985: {
+    options = {
+        hoist_props: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        let a = { p: 42 };
+        console.log(function() {
+            a;
+        }());
+    }
+    expect: {
+        "use strict";
+        let a = { p: 42 };
+        console.log(function() {
+            a;
+        }());
+    }
+    expect_stdout: "undefined"
+    node_version: ">=4"
+}
+
+issue_5240: {
+    options = {
+        inline: true,
+    }
+    input: {
+        "use strict";
+        function f() {
+            if (console) {
+                let g = function() {
+                    e;
+                }, e;
+                (function() {
+                    if (console) {
+                        console.log(e);
+                        var e = "FAIL";
+                    }
+                })(console.log(e));
+            }
+        }
+        f();
+    }
+    expect: {
+        "use strict";
+        function f() {
+            if (console) {
+                let g = function() {
+                    e;
+                }, e;
+                (function() {
+                    if (console) {
+                        console.log(e);
+                        var e = "FAIL";
+                    }
+                })(console.log(e));
+            }
+        }
+        f();
+    }
+    expect_stdout: [
+        "undefined",
+        "undefined",
+    ]
+    node_version: ">=4"
+}
+
+issue_5254: {
+    options = {
+        inline: true,
+        toplevel: true,
+    }
+    input: {
+        "use strict";
+        do {
+            (function() {
+                let a = console.log;
+                a && a("foo");
+            })();
+        } while (console.log("bar"));
+    }
+    expect: {
+        "use strict";
+        do {
+            let a = console.log;
+            a && a("foo");
+        } while (console.log("bar"));
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+    ]
+    node_version: ">=4"
+}
+
+issue_5260: {
+    options = {
+        collapse_vars: true,
+    }
+    input: {
+        "use strict";
+        var a = "foo", o;
+        while (console.log("bar"));
+        o = {
+            baz: function(b) {
+                console.log(a, b);
+            },
+        };
+        for (let a in o)
+            o[a](a);
+    }
+    expect: {
+        "use strict";
+        var a = "foo", o;
+        while (console.log("bar"));
+        o = {
+            baz: function(b) {
+                console.log(a, b);
+            },
+        };
+        for (let a in o)
+            o[a](a);
+    }
+    expect_stdout: [
+        "bar",
+        "foo baz",
+    ]
+    node_version: ">=4"
+}
+
+issue_5319: {
+    options = {
+        collapse_vars: true,
+        merge_vars: true,
+    }
+    input: {
+        "use strict";
+        (function(a, c) {
+            var b = a, c = b;
+            {
+                let a = c;
+                console.log(c());
+            }
+        })(function() {
+            return "PASS";
+        });
+    }
+    expect: {
+        "use strict";
+        (function(a, c) {
+            var b = a, c;
+            {
+                let a = c = b;
+                console.log(c());
+            }
+        })(function() {
+            return "PASS";
+        });
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5338: {
+    options = {
+        unused: true,
+    }
+    input: {
+        "use strict";
+        let a = a;
+    }
+    expect: {
+        "use strict";
+        a;
+        let a;
+    }
+    expect_stdout: ReferenceError("a is not defined")
+    node_version: ">=4"
+}
