@@ -193,9 +193,9 @@ evaluate: {
         } while (false);
     }
     expect: {
-        for(;;)
+        for (;;)
             a();
-        for(;;)
+        for (;;)
             c();
         d();
     }
@@ -265,7 +265,7 @@ issue_1532_2: {
 issue_186: {
     beautify = {
         beautify: false,
-        ie8: false,
+        ie: false,
     }
     input: {
         var x = 3;
@@ -284,7 +284,7 @@ issue_186: {
 issue_186_ie8: {
     beautify = {
         beautify: false,
-        ie8: true,
+        ie: true,
     }
     input: {
         var x = 3;
@@ -303,7 +303,7 @@ issue_186_ie8: {
 issue_186_beautify: {
     beautify = {
         beautify: true,
-        ie8: false,
+        ie: false,
     }
     input: {
         var x = 3;
@@ -330,7 +330,7 @@ issue_186_beautify: {
 issue_186_beautify_ie8: {
     beautify = {
         beautify: true,
-        ie8: true,
+        ie: true,
     }
     input: {
         var x = 3;
@@ -360,7 +360,7 @@ issue_186_braces: {
     beautify = {
         beautify: false,
         braces: true,
-        ie8: false,
+        ie: false,
     }
     input: {
         var x = 3;
@@ -380,7 +380,7 @@ issue_186_braces_ie8: {
     beautify = {
         beautify: false,
         braces: true,
-        ie8: true,
+        ie: true,
     }
     input: {
         var x = 3;
@@ -400,7 +400,7 @@ issue_186_beautify_braces: {
     beautify = {
         beautify: true,
         braces: true,
-        ie8: false,
+        ie: false,
     }
     input: {
         var x = 3;
@@ -432,7 +432,7 @@ issue_186_beautify_braces_ie8: {
     beautify = {
         beautify: true,
         braces: true,
-        ie8: true,
+        ie: true,
     }
     input: {
         var x = 3;
@@ -829,6 +829,18 @@ empty_for_in_prop_init: {
 }
 
 for_of: {
+    input: {
+        var a = [ "PASS", 42 ];
+        a.p = "FAIL";
+        for (a of (null, a))
+            console.log(a);
+    }
+    expect_exact: 'var a=["PASS",42];a.p="FAIL";for(a of(null,a))console.log(a);'
+    expect_stdout: true
+    node_version: ">=0.12"
+}
+
+for_async_of: {
     input: {
         var async = [ "PASS", 42 ];
         async.p = "FAIL";
@@ -837,7 +849,21 @@ for_of: {
     }
     expect_exact: 'var async=["PASS",42];async.p="FAIL";for(async of(null,async))console.log(async);'
     expect_stdout: true
-    node_version: ">=0.12"
+    node_version: ">=0.12 <16"
+}
+
+for_of_regexp: {
+    input: {
+        for (var a of /foo/);
+    }
+    expect_exact: "for(var a of/foo/);"
+}
+
+for_await_of_regexp: {
+    input: {
+        for await (var a of /foo/);
+    }
+    expect_exact: "for await(var a of/foo/);"
 }
 
 issue_3631_1: {
