@@ -17,7 +17,7 @@ dot_properties: {
         properties: true,
     }
     beautify = {
-        ie8: true,
+        ie: true,
     }
     input: {
         a["foo"] = "bar";
@@ -43,7 +43,7 @@ dot_properties_es5: {
         properties: true,
     }
     beautify = {
-        ie8: false,
+        ie: false,
     }
     input: {
         a["foo"] = "bar";
@@ -1446,3 +1446,135 @@ issue_4831_2: {
     expect_stdout: "PASS"
     node_version: ">=4"
 }
+
+issue_4888: {
+    options = {
+        properties: true,
+    }
+    input: {
+        console.log(typeof {
+            __proto__: 42,
+        }.__proto__);
+    }
+    expect: {
+        console.log(typeof {
+            __proto__: 42,
+        }.__proto__);
+    }
+    expect_stdout: "object"
+}
+
+issue_5093: {
+    beautify = {
+        keep_quoted_props: true,
+    }
+    input: {
+        console.log({
+            a: true,
+            '42': "PASS",
+            "null": [],
+        }[6 * 7]);
+    }
+    expect_exact: 'console.log({a:true,"42":"PASS","null":[]}[6*7]);'
+    expect_stdout: "PASS"
+}
+
+issue_5093_quote_keys: {
+    beautify = {
+        keep_quoted_props: true,
+        quote_keys: true,
+    }
+    input: {
+        console.log({
+            a: true,
+            '42': "PASS",
+            "null": [],
+        }[6 * 7]);
+    }
+    expect_exact: 'console.log({"a":true,"42":"PASS","null":[]}[6*7]);'
+    expect_stdout: "PASS"
+}
+
+issue_5093_quote_style: {
+    beautify = {
+        keep_quoted_props: true,
+        quote_style: 3,
+    }
+    input: {
+        console.log({
+            a: true,
+            '42': "PASS",
+            "null": [],
+        }[6 * 7]);
+    }
+    expect_exact: 'console.log({a:true,\'42\':"PASS","null":[]}[6*7]);'
+    expect_stdout: "PASS"
+}
+
+object_methods: {
+    options = {
+        properties: true,
+    }
+    input: {
+        ({
+            p() {
+                console.log("FAIL 1");
+            },
+            *q() {
+                console.log("FAIL 2");
+            },
+            async r() {
+                console.log("FAIL 3");
+            },
+            async *s() {
+                console.log("PASS");
+            },
+        }).s().next();
+    }
+    expect: {
+        [
+            () => {
+                console.log("FAIL 1");
+            },
+            function*() {
+                console.log("FAIL 2");
+            },
+            async () => {
+                console.log("FAIL 3");
+            },
+            async function*() {
+                console.log("PASS");
+            },
+        ][3]().next();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10"
+}
+
+issue_5177: {
+    options = {
+        properties: true,
+    }
+    input: {
+        var a = "FAIL";
+        var o = { a: "PASS" };
+        o.p = {
+            q() {
+                return this.a;
+            },
+        }.q;
+        console.log(o.p());
+    }
+    expect: {
+        var a = "FAIL";
+        var o = { a: "PASS" };
+        o.p = {
+            q() {
+                return this.a;
+            },
+        }.q;
+        console.log(o.p());
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
