@@ -51,7 +51,7 @@ arrow_destructured_object_1: {
     }
     expect_exact: "var f=({...a})=>a,o=f({PASS:42});for(var k in o)console.log(k,o[k]);"
     expect_stdout: "PASS 42"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 arrow_destructured_object_2: {
@@ -62,7 +62,7 @@ arrow_destructured_object_2: {
     }
     expect_exact: "var f=({FAIL:a,...b})=>b,o=f({PASS:42,FAIL:null});for(var k in o)console.log(k,o[k]);"
     expect_stdout: "PASS 42"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 arrow_destructured_object_3: {
@@ -79,7 +79,7 @@ arrow_destructured_object_3: {
         "2 S",
         "3 S",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 funarg_1: {
@@ -131,7 +131,7 @@ destructured_object_1: {
     }
     expect_exact: 'var{...a}=["FAIL","PASS",42];console.log(a[1],a[2]);'
     expect_stdout: "PASS 42"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 destructured_object_2: {
@@ -141,7 +141,7 @@ destructured_object_2: {
     }
     expect_exact: 'var{0:a,...b}=["FAIL","PASS",42];console.log(b[1],b[2]);'
     expect_stdout: "PASS 42"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 drop_fargs: {
@@ -231,7 +231,7 @@ reduce_destructured_object: {
         console.log(a[0]);
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 retain_destructured_array: {
@@ -244,7 +244,7 @@ retain_destructured_array: {
         console.log.apply(console, b);
     }
     expect: {
-        var [ , ...b ] = [ "FAIL", "PASS", 42 ];
+        var [ ...b ] = [ "PASS", 42 ];
         console.log.apply(console, b);
     }
     expect_stdout: "PASS 42"
@@ -270,7 +270,7 @@ retain_destructured_object_1: {
         "1 PASS",
         "2 42",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 retain_destructured_object_2: {
@@ -284,7 +284,7 @@ retain_destructured_object_2: {
             console.log(k, b[k]);
     }
     expect: {
-        var { foo: [], ...b } = { foo: [ "FAIL" ], bar: "PASS", baz: 42 };
+        var { foo: {}, ...b } = { foo: 0, bar: "PASS", baz: 42 };
         for (var k in b)
             console.log(k, b[k]);
     }
@@ -292,7 +292,7 @@ retain_destructured_object_2: {
         "bar PASS",
         "baz 42",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 retain_funarg_destructured_array_1: {
@@ -344,11 +344,12 @@ retain_funarg_destructured_object_1: {
         console.log((({ ...a }) => a)([ "PASS" ])[0]);
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 retain_funarg_destructured_object_2: {
     options = {
+        keep_fargs: false,
         unused: true,
     }
     input: {
@@ -359,10 +360,10 @@ retain_funarg_destructured_object_2: {
     expect: {
         console.log(function({ p: a, ... b }) {
             return b;
-        }({ p: "FAIL" }).p || "PASS");
+        }({}).p || "PASS");
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 drop_unused_call_args_1: {
@@ -407,6 +408,24 @@ drop_unused_call_args_2: {
     node_version: ">=6"
 }
 
+maintain_position: {
+    options = {
+        unused: true,
+    }
+    input: {
+        A = "FAIL";
+        var [ , ...a ] = [ A, "PASS" ];
+        console.log(a[0]);
+    }
+    expect: {
+        A = "FAIL";
+        var [ , ...a ] = [ A, "PASS" ];
+        console.log(a[0]);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 merge_funarg: {
     options = {
         merge_vars: true,
@@ -418,9 +437,9 @@ merge_funarg: {
         })();
     }
     expect: {
-        (function(...b) {
-            var b = b.length;
-            console.log(b);
+        (function(...a) {
+            var a = a.length;
+            console.log(a);
         })();
     }
     expect_stdout: "0"
@@ -438,9 +457,9 @@ merge_funarg_destructured_array: {
         })([]);
     }
     expect: {
-        (function([ ...b ]) {
-            var b = b.length;
-            console.log(b);
+        (function([ ...a ]) {
+            var a = a.length;
+            console.log(a);
         })([]);
     }
     expect_stdout: "0"
@@ -458,13 +477,13 @@ merge_funarg_destructured_object: {
         })([ "PASS" ]);
     }
     expect: {
-        (function({ ...b }) {
-            var b = b[0];
-            console.log(b);
+        (function({ ...a }) {
+            var a = a[0];
+            console.log(a);
         })([ "PASS" ]);
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 keep_arguments: {
@@ -491,7 +510,7 @@ drop_rest_array: {
         rests: true,
     }
     input: {
-        var [ ...[ a ]] = [ "PASS" ];
+        var [ ...[ a ] ] = [ "PASS" ];
         console.log(a);
     }
     expect: {
@@ -542,6 +561,100 @@ drop_rest_lambda: {
     node_version: ">=6"
 }
 
+keep_rest_array: {
+    options = {
+        rests: true,
+    }
+    input: {
+        var [ ...[ ...a ] ] = "PASS";
+        console.log(a.join(""));
+    }
+    expect: {
+        var [ ...a ] = "PASS";
+        console.log(a.join(""));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+keep_rest_arrow: {
+    options = {
+        arrows: true,
+        keep_fargs: false,
+        reduce_vars: true,
+        rests: true,
+    }
+    input: {
+        console.log(((...[ ...a ]) => a.join(""))("PASS"));
+    }
+    expect: {
+        console.log(((...a) => a.join(""))("PASS"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+keep_rest_lambda_1: {
+    options = {
+        keep_fargs: false,
+        reduce_vars: true,
+        rests: true,
+        toplevel: true,
+    }
+    input: {
+        function f(...[ ...a ]) {
+            return a.join("");
+        }
+        console.log(f("PASS"), f([ 42 ]));
+    }
+    expect: {
+        function f(...a) {
+            return a.join("");
+        }
+        console.log(f("PASS"), f([ 42 ]));
+    }
+    expect_stdout: "PASS 42"
+    node_version: ">=6"
+}
+
+keep_rest_lambda_2: {
+    options = {
+        unused: true,
+    }
+    input: {
+        function f(...[ ...a ]) {
+            return a.join("");
+        }
+        console.log(f("PASS"), f([ 42 ]));
+    }
+    expect: {
+        function f(...[ ...a ]) {
+            return a.join("");
+        }
+        console.log(f("PASS"), f([ 42 ]));
+    }
+    expect_stdout: "PASS 42"
+    node_version: ">=6"
+}
+
+drop_new_function: {
+    options = {
+        side_effects: true,
+    }
+    input: {
+        new function(...{
+            [console.log("PASS")]: a,
+        }) {}();
+    }
+    expect: {
+        void ([ ... {
+            [console.log("PASS")]: [].e,
+        }] = []);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 issue_4525_1: {
     options = {
         arguments: true,
@@ -647,6 +760,78 @@ issue_4544_2: {
     node_version: ">=6"
 }
 
+issue_4560_1: {
+    options = {
+        evaluate: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = 0;
+        (function(...{
+            [a++]: {},
+        }) {})(2);
+        console.log(a);
+    }
+    expect: {
+        var a = 0;
+        (function(...{
+            [a++]: {},
+        }) {})(2);
+        console.log(a);
+    }
+    expect_stdout: "1"
+    node_version: ">=6"
+}
+
+issue_4560_2: {
+    options = {
+        reduce_vars: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        var a = 0;
+        (function(...{
+            [a++]: {},
+        }) {})(2);
+        console.log(a);
+    }
+    expect: {
+        var a = 0;
+        (function(...{
+            [a++]: {},
+        }) {})(2);
+        console.log(a);
+    }
+    expect_stdout: "1"
+    node_version: ">=6"
+}
+
+issue_4560_3: {
+    options = {
+        collapse_vars: true,
+        reduce_vars: true,
+        toplevel: true,
+    }
+    input: {
+        var a = 0, b;
+        [ ...{
+            [a++]: b,
+        } ] = [ "PASS" ];
+        console.log(b);
+    }
+    expect: {
+        var a = 0, b;
+        [ ...{
+            [a++]: b,
+        } ] = [ "PASS" ];
+        console.log(b);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 issue_4562: {
     options = {
         evaluate: true,
@@ -667,7 +852,7 @@ issue_4562: {
 issue_4575: {
     options = {
         collapse_vars: true,
-        ie8: true,
+        ie: true,
         reduce_vars: true,
         rests: true,
         unused: true,
@@ -682,9 +867,9 @@ issue_4575: {
     }
     expect: {
         (function(a) {
-            (function a(...d) {
+            (function(d) {
                 console.log(d.length);
-            })();
+            })([]);
         })();
     }
     expect_stdout: "0"
@@ -777,10 +962,308 @@ issue_4666: {
     expect: {
         var a = 0, b = 0;
         var o = (c => +a + c)([ b ]);
-        for(var k in o)
+        for (var k in o)
             b++;
         console.log(1, b);
     }
     expect_stdout: "1 2"
     node_version: ">=6"
 }
+
+issue_5089_1: {
+    options = {
+        unused: true,
+    }
+    input: {
+        var {
+            p: [] = 42,
+            ...o
+        } = {
+            p: [],
+        };
+        console.log(o.p);
+    }
+    expect: {
+        var {
+            p: {},
+            ...o
+        } = {
+            p: 0,
+        };
+        console.log(o.p);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=8.3.0"
+}
+
+issue_5089_2: {
+    options = {
+        pure_getters: "strict",
+        unused: true,
+    }
+    input: {
+        var {
+            p: {} = null,
+            ...o
+        } = {
+            p: {},
+        };
+        console.log(o.p);
+    }
+    expect: {
+        var {
+            p: {},
+            ...o
+        } = {
+            p: 0,
+        };
+        console.log(o.p);
+    }
+    expect_stdout: "undefined"
+    node_version: ">=8.3.0"
+}
+
+issue_5100_1: {
+    options = {
+        passes: 2,
+        pure_getters: "strict",
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        [ {
+            p: {},
+            ...a
+        } ] = [ {
+            p: {
+                q: a,
+            } = 42,
+            r: "PASS",
+        } ];
+        console.log(a.r);
+    }
+    expect: {
+        var a;
+        ({
+            p: {},
+            ...a
+        } = [ {
+            p: [ a = 42["q"] ],
+            r: "PASS",
+        } ][0]);
+        console.log(a.r);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=8.3.0"
+}
+
+issue_5100_2: {
+    options = {
+        passes: 2,
+        pure_getters: "strict",
+        side_effects: true,
+        unused: true,
+    }
+    input: {
+        var a;
+        [ {
+            p: {},
+            ...a
+        } ] = [ {
+            p: (console.log("PASS"), {
+                q: a,
+            } = 42),
+        } ];
+    }
+    expect: {
+        var a;
+        ({
+            p: {},
+            ...a
+        } = [ {
+            p: [ console.log("PASS"), a = 42["q"] ],
+        } ][0]);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=10.22.0"
+}
+
+issue_5108: {
+    options = {
+        evaluate: true,
+        keep_fargs: false,
+        reduce_vars: true,
+        rests: true,
+        unsafe: true,
+        unused: true,
+    }
+    input: {
+        console.log(function([ ...[ a ] ]) {
+            return a;
+        }([ "PASS", "FAIL" ]));
+    }
+    expect: {
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5128_1: {
+    options = {
+        inline: true,
+    }
+    input: {
+        console.log(function() {
+            return function f(...[ a ]) {
+                return a;
+            }("PASS");
+        }());
+    }
+    expect: {
+        console.log(function f(...[ a ]) {
+            return a;
+        }("PASS"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5128_2: {
+    options = {
+        inline: true,
+        keep_fnames: true,
+        unused: true,
+    }
+    input: {
+        console.log(function() {
+            return function f(...[ a ]) {
+                return a;
+            }("PASS");
+        }());
+    }
+    expect: {
+        console.log(function f(...[ a ]) {
+            return a;
+        }("PASS"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5165_1: {
+    options = {
+        conditionals: true,
+        dead_code: true,
+        evaluate: true,
+        reduce_vars: true,
+        rests: true,
+        side_effects: true,
+        switches: true,
+        unsafe: true,
+    }
+    input: {
+        console.log(function([ ...a ]) {
+            switch (a) {
+              case a:
+                return "PASS";
+            }
+        }([]));
+    }
+    expect: {
+        console.log(function([ ...a ]) {
+            return "PASS";
+        }([]));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5165_2: {
+    options = {
+        conditionals: true,
+        dead_code: true,
+        evaluate: true,
+        reduce_vars: true,
+        rests: true,
+        side_effects: true,
+        switches: true,
+        unsafe: true,
+    }
+    input: {
+        console.log(function(...a) {
+            switch (a) {
+              case a:
+                return "PASS";
+            }
+        }());
+    }
+    expect: {
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5246_1: {
+    options = {
+        keep_fargs: false,
+        reduce_vars: true,
+        rests: true,
+        unused: true,
+    }
+    input: {
+        console.log(typeof function([ , ...a ]) {
+            return this && a;
+        }([ , function(){} ])[0]);
+    }
+    expect: {
+        console.log(typeof function() {
+            return this && [ function(){} ];
+        }()[0]);
+    }
+    expect_stdout: "function"
+    node_version: ">=6"
+}
+
+issue_5246_2: {
+    options = {
+        keep_fargs: false,
+        reduce_vars: true,
+        rests: true,
+        toplevel: true,
+        unused: true,
+    }
+    input: {
+        A = [ , "PASS", "FAIL" ];
+        var  [ , ...a ] = [ ... A ];
+        console.log(a[0]);
+    }
+    expect: {
+        A = [ , "PASS", "FAIL" ];
+        var  [ , ...a ] = [ ... A ];
+        console.log(a[0]);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5246_3: {
+    options = {
+        keep_fargs: false,
+        unused: true,
+    }
+    input: {
+        (function f(...[ [ a ] ]) {
+            console.log(a);
+        })([ "PASS" ]);
+    }
+    expect: {
+        (function(...[ a ]) {
+            console.log(a);
+        })([ "PASS" ][0]);
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
