@@ -147,7 +147,7 @@ dont_inline: {
     node_version: ">=6"
 }
 
-do_inline: {
+do_inline_1: {
     options = {
         inline: true,
         spreads: true,
@@ -164,6 +164,46 @@ do_inline: {
     node_version: ">=6"
 }
 
+do_inline_2: {
+    options = {
+        inline: true,
+        side_effects: true,
+    }
+    input: {
+        (function() {
+            (function() {
+                console.log("PASS");
+            })(..."");
+        })();
+    }
+    expect: {
+        [] = [ ..."" ],
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+do_inline_3: {
+    options = {
+        if_return: true,
+        inline: true,
+    }
+    input: {
+        (function() {
+            (function() {
+                while (console.log("PASS"));
+            })(..."");
+        })();
+    }
+    expect: {
+        var [] = [ ..."" ];
+        while (console.log("PASS"));
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
 drop_empty_call_1: {
     options = {
         side_effects: true,
@@ -201,7 +241,29 @@ drop_empty_call_2: {
     node_version: ">=6"
 }
 
-convert_hole: {
+convert_hole_array: {
+    options = {
+        spreads: true,
+    }
+    input: {
+        [ ...[ "PASS", , 42 ] ].forEach(function(a) {
+            console.log(a);
+        });
+    }
+    expect: {
+        [ "PASS", void 0, 42 ].forEach(function(a) {
+            console.log(a);
+        });
+    }
+    expect_stdout: [
+        "PASS",
+        "undefined",
+        "42",
+    ]
+    node_version: ">=6"
+}
+
+convert_hole_call: {
     options = {
         spreads: true,
     }
@@ -341,7 +403,7 @@ convert_setter: {
             console.log(k, o[k]);
     }
     expect_stdout: "PASS undefined"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 keep_getter_1: {
@@ -370,7 +432,7 @@ keep_getter_1: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 keep_getter_2: {
@@ -399,7 +461,7 @@ keep_getter_2: {
         "foo",
         "bar",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 keep_getter_3: {
@@ -429,7 +491,7 @@ keep_getter_3: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 keep_getter_4: {
@@ -460,7 +522,7 @@ keep_getter_4: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 keep_accessor: {
@@ -508,7 +570,7 @@ keep_accessor: {
         "q undefined",
         "r null",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 object_key_order_1: {
@@ -538,7 +600,7 @@ object_key_order_1: {
         "a 3",
         "b 2",
     ]
-    node_version: ">=8 <=10"
+    node_version: ">=8.3.0 <=10"
 }
 
 object_key_order_2: {
@@ -568,7 +630,7 @@ object_key_order_2: {
         "a 3",
         "b 2",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 object_key_order_3: {
@@ -598,7 +660,7 @@ object_key_order_3: {
         "a 3",
         "b 2",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 object_key_order_4: {
@@ -628,7 +690,7 @@ object_key_order_4: {
         "a 3",
         "b 2",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 object_spread_array: {
@@ -654,7 +716,7 @@ object_spread_array: {
         "0 foo",
         "1 bar",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 object_spread_string: {
@@ -681,7 +743,7 @@ object_spread_string: {
         "1 o",
         "2 o",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 unused_var_side_effects: {
@@ -711,7 +773,7 @@ unused_var_side_effects: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 unsafe_join_1: {
@@ -793,7 +855,7 @@ issue_4329: {
         }[0]);
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 issue_4331: {
@@ -871,7 +933,7 @@ issue_4345: {
         }[42]);
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 issue_4361: {
@@ -901,7 +963,7 @@ issue_4361: {
         "foo",
         "undefined",
     ]
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 issue_4363: {
@@ -922,7 +984,7 @@ issue_4363: {
         });
     }
     expect_stdout: "PASS"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
 }
 
 issue_4556: {
@@ -945,78 +1007,6 @@ issue_4556: {
     node_version: ">=6"
 }
 
-issue_4560_1: {
-    options = {
-        evaluate: true,
-        reduce_vars: true,
-        toplevel: true,
-    }
-    input: {
-        var a = 0;
-        (function(...{
-            [a++]: {},
-        }) {})(2);
-        console.log(a);
-    }
-    expect: {
-        var a = 0;
-        (function(...{
-            [a++]: {},
-        }) {})(2);
-        console.log(a);
-    }
-    expect_stdout: "1"
-    node_version: ">=6"
-}
-
-issue_4560_2: {
-    options = {
-        reduce_vars: true,
-        toplevel: true,
-        unused: true,
-    }
-    input: {
-        var a = 0;
-        (function(...{
-            [a++]: {},
-        }) {})(2);
-        console.log(a);
-    }
-    expect: {
-        var a = 0;
-        (function(...{
-            [a++]: {},
-        }) {})(2);
-        console.log(a);
-    }
-    expect_stdout: "1"
-    node_version: ">=6"
-}
-
-issue_4560_3: {
-    options = {
-        collapse_vars: true,
-        reduce_vars: true,
-        toplevel: true,
-    }
-    input: {
-        var a = 0, b;
-        [ ...{
-            [a++]: b,
-        } ] = [ "PASS" ];
-        console.log(b);
-    }
-    expect: {
-        var a = 0, b;
-        [ ...{
-            [a++]: b,
-        } ] = [ "PASS" ];
-        console.log(b);
-    }
-    expect_stdout: "PASS"
-    node_version: ">=6"
-}
-
 issue_4614: {
     options = {
         pure_getters: "strict",
@@ -1066,5 +1056,113 @@ issue_4849: {
         }()));
     }
     expect_stdout: "object"
-    node_version: ">=8"
+    node_version: ">=8.3.0"
+}
+
+issue_4882_1: {
+    options = {
+        objects: true,
+        spreads: true,
+    }
+    input: {
+        var o = {
+            p: "PASS",
+            ... {
+                __proto__: {
+                    p: "FAIL 1",
+                    q: "FAIL 2",
+                },
+            },
+        };
+        console.log(o.p);
+        console.log(o.q);
+    }
+    expect: {
+        var o = {
+            p: "PASS",
+        };
+        console.log(o.p);
+        console.log(o.q);
+    }
+    expect_stdout: [
+        "PASS",
+        "undefined",
+    ]
+    node_version: ">=8.3.0"
+}
+
+issue_4882_2: {
+    options = {
+        objects: true,
+        spreads: true,
+    }
+    input: {
+        console.log(null == Object.getPrototypeOf({
+            ... {
+                __proto__: (console.log(42), null),
+            },
+        }) ? "FAIL" : "PASS");
+    }
+    expect: {
+        console.log(null == Object.getPrototypeOf({
+            ... {
+                __proto__: (console.log(42), null),
+            },
+        }) ? "FAIL" : "PASS");
+    }
+    expect_stdout: [
+        "42",
+        "PASS",
+    ]
+    node_version: ">=8.3.0"
+}
+
+issue_4882_3: {
+    options = {
+        objects: true,
+        spreads: true,
+    }
+    input: {
+        var o = {
+            __proto__: { p: 42 },
+            ... {
+                set __proto__(v) {},
+            },
+        };
+        console.log(o.__proto__ === Object.getPrototypeOf(o) ? "FAIL" : "PASS");
+        console.log(o.p);
+    }
+    expect: {
+        var o = {
+            __proto__: { p: 42 },
+            ["__proto__"]: void 0,
+        };
+        console.log(o.__proto__ === Object.getPrototypeOf(o) ? "FAIL" : "PASS");
+        console.log(o.p);
+    }
+    expect_stdout: [
+        "PASS",
+        "42",
+    ]
+    node_version: ">=8.3.0"
+}
+
+issue_5006: {
+    options = {
+        arguments: true,
+    }
+    input: {
+        console.log(function(b, c) {
+            c = "FAIL 2";
+            return arguments[1];
+        }(...[], "FAIL 1") || "PASS");
+    }
+    expect: {
+        console.log(function(b, c) {
+            c = "FAIL 2";
+            return arguments[1];
+        }(...[], "FAIL 1") || "PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
 }
