@@ -96,7 +96,7 @@ pause_resume: {
     node_version: ">=4"
 }
 
-arrow_yield: {
+arrow_yield_1: {
     input: {
         yield = "PASS";
         console.log(function*() {
@@ -108,6 +108,18 @@ arrow_yield: {
     node_version: ">=4"
 }
 
+arrow_yield_2: {
+    input: {
+        console.log(typeof function *() {
+            // Syntax error on Node.js v6+
+            return (yield) => {};
+        }().next().value);
+    }
+    expect_exact: "console.log(typeof function*(){return(yield)=>{}}().next().value);"
+    expect_stdout: "function"
+    node_version: "4"
+}
+
 for_of: {
     input: {
         function* f() {
@@ -150,6 +162,27 @@ for_await_of: {
     node_version: ">=10"
 }
 
+comment_newline: {
+    beautify = {
+        comments: "all",
+    }
+    input: {
+        console.log(function*() {
+            yield (
+                /* */
+                "PASS"
+            );
+        }().next().value);
+    }
+    expect_exact: [
+        "console.log(function*(){",
+        "/* */",
+        'yield"PASS"}().next().value);',
+    ]
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 collapse_vars_1: {
     options = {
         collapse_vars: true,
@@ -716,7 +749,7 @@ lift_sequence: {
     node_version: ">=4"
 }
 
-inline_nested_yield: {
+inline_nested: {
     options = {
         inline: true,
         sequences: true,
@@ -751,6 +784,118 @@ inline_nested_yield: {
     node_version: ">=4"
 }
 
+inline_nested_async: {
+    options = {
+        awaits: true,
+        inline: true,
+        sequences: true,
+        side_effects: true,
+        yields: true,
+    }
+    input: {
+        console.log("foo");
+        var a = async function*() {
+            console.log(await(yield* async function*() {
+                yield {
+                    then: r => r("bar"),
+                };
+                return "baz";
+            }()));
+        }();
+        console.log("moo");
+        a.next().then(function f(b) {
+            console.log(b.value);
+            b.done || a.next().then(f);
+        });
+        console.log("moz");
+    }
+    expect: {
+        console.log("foo");
+        var a = async function*() {
+            console.log((yield {
+                then: r => r("bar"),
+            }, await "baz"));
+        }();
+        console.log("moo"),
+        a.next().then(function f(b) {
+            console.log(b.value),
+            b.done || a.next().then(f);
+        }),
+        console.log("moz");
+    }
+    expect_stdout: [
+        "foo",
+        "moo",
+        "moz",
+        "bar",
+        "baz",
+        "undefined",
+    ]
+    node_version: ">=10"
+}
+
+inline_nested_block: {
+    options = {
+        if_return: true,
+        inline: true,
+        yields: true,
+    }
+    input: {
+        var a = function*() {
+            yield* function*() {
+                for (var a of [ "foo", "bar" ])
+                    yield a;
+                return "FAIL";
+            }();
+        }(), b;
+        do {
+            b = a.next();
+            console.log(b.value);
+        } while (!b.done);
+    }
+    expect: {
+        var a = function*() {
+            for (var a of [ "foo", "bar" ])
+                yield a;
+            "FAIL";
+        }(), b;
+        do {
+            b = a.next();
+            console.log(b.value);
+        } while (!b.done);
+    }
+    expect_stdout: [
+        "foo",
+        "bar",
+        "undefined",
+    ]
+    node_version: ">=4"
+}
+
+dont_inline_nested: {
+    options = {
+        inline: true,
+    }
+    input: {
+        var yield = "PASS";
+        (function*() {
+            (function() {
+                console.log(yield);
+            })();
+        })().next();
+    }
+    expect: {
+        var yield = "PASS";
+        (function*() {
+            (function() {
+                console.log(yield);
+            })();
+        })().next();
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
 drop_body: {
     options = {
         side_effects: true,
@@ -831,10 +976,10 @@ issue_4454_2: {
         f("PASS");
     }
     expect: {
-        function f(b) {
-            (function*(c = console.log(b)) {})();
-            var b = 42..toString();
-            console.log(b);
+        function f(a) {
+            (function*(c = console.log(a)) {})();
+            var a = 42..toString();
+            console.log(a);
         }
         f("PASS");
     }
@@ -937,7 +1082,7 @@ issue_4639_1: {
         }().next().value());
     }
     expect_stdout: "PASS"
-    node_version: ">=4"
+    node_version: ">=4 <7 || >=8.7.0"
 }
 
 issue_4639_2: {
@@ -1062,3 +1207,247 @@ issue_4769_2: {
     expect_stdout: "PASS"
     node_version: ">=6"
 }
+
+issue_5019_1: {
+    options = {
+        dead_code: true,
+    }
+    input: {
+        (function(a) {
+            return a = function*() {
+                console.log(typeof a);
+            }();
+        })().next();
+    }
+    expect: {
+        (function(a) {
+            return a = function*() {
+                console.log(typeof a);
+            }();
+        })().next();
+    }
+    expect_stdout: "object"
+    node_version: ">=4"
+}
+
+issue_5019_2: {
+    options = {
+        inline: true,
+        toplevel: true,
+    }
+    input: {
+        var a = [];
+        for (var b in "foo")
+            a.push(function(c) {
+                return function*() {
+                    console.log(c);
+                }();
+            }(b));
+        a.map(function(d) {
+            return d.next();
+        });
+    }
+    expect: {
+        var a = [];
+        for (var b in "foo")
+            a.push(function(c) {
+                return function*() {
+                    console.log(c);
+                }();
+            }(b));
+        a.map(function(d) {
+            return d.next();
+        });
+    }
+    expect_stdout: [
+        "0",
+        "1",
+        "2",
+    ]
+    node_version: ">=4"
+}
+
+issue_5032_normal: {
+    options = {
+        merge_vars: true,
+        webkit: false,
+    }
+    input: {
+        function log(value) {
+            console.log(value);
+            return value;
+        }
+        function *f(a) {
+            var b = log(a), c = b;
+            log(b);
+            log(c);
+        }
+        f("PASS").next();
+    }
+    expect: {
+        function log(value) {
+            console.log(value);
+            return value;
+        }
+        function *f(a) {
+            var a = log(a), c = a;
+            log(a);
+            log(c);
+        }
+        f("PASS").next();
+    }
+    expect_stdout: [
+        "PASS",
+        "PASS",
+        "PASS",
+    ]
+    node_version: ">=4"
+}
+
+issue_5032_webkit: {
+    options = {
+        merge_vars: true,
+        webkit: true,
+    }
+    input: {
+        function log(value) {
+            console.log(value);
+            return value;
+        }
+        function *f(a) {
+            var b = log(a), c = b;
+            log(b);
+            log(c);
+        }
+        f("PASS").next();
+    }
+    expect: {
+        function log(value) {
+            console.log(value);
+            return value;
+        }
+        function *f(a) {
+            var b = log(a), c = b;
+            log(b);
+            log(c);
+        }
+        f("PASS").next();
+    }
+    expect_stdout: [
+        "PASS",
+        "PASS",
+        "PASS",
+    ]
+    node_version: ">=4"
+}
+
+issue_5034: {
+    options = {
+        functions: true,
+        reduce_vars: true,
+        unused: true,
+    }
+    input: {
+        console.log(function() {
+            var yield = function f() {
+                return function*() {
+                    return f;
+                };
+            };
+            return yield()().next().value === yield;
+        }() ? "PASS" : "FAIL");
+    }
+    expect: {
+        console.log(function() {
+            var yield = function f() {
+                return function*() {
+                    return f;
+                };
+            };
+            return yield()().next().value === yield;
+        }() ? "PASS" : "FAIL");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=4"
+}
+
+issue_5076_1: {
+    options = {
+        evaluate: true,
+        hoist_vars: true,
+        keep_fargs: false,
+        pure_getters: "strict",
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+        yields: true,
+    }
+    input: {
+        var a;
+        console.log("PASS");
+        var b = function*({
+            p: {},
+        }) {}({
+            p: { a } = 42,
+        });
+    }
+    expect: {
+        var a;
+        console.log("PASS"),
+        a = 42["a"];
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5076_2: {
+    options = {
+        evaluate: true,
+        hoist_vars: true,
+        keep_fargs: false,
+        passes: 2,
+        pure_getters: "strict",
+        sequences: true,
+        side_effects: true,
+        toplevel: true,
+        unused: true,
+        yields: true,
+    }
+    input: {
+        var a;
+        console.log("PASS");
+        var b = function*({
+            p: {},
+        }) {}({
+            p: { a } = 42,
+        });
+    }
+    expect: {
+        console.log("PASS");
+    }
+    expect_stdout: "PASS"
+    node_version: ">=6"
+}
+
+issue_5177: {
+    options = {
+        properties: true,
+    }
+    input: {
+        console.log(typeof function*() {
+            return {
+                p(yield) {},
+            }.p;
+        }().next().value);
+    }
+    expect: {
+        console.log(typeof function*() {
+            return {
+                p(yield) {},
+            }.p;
+        }().next().value);
+    }
+    expect_stdout: "function"
+    node_version: ">=4"
+}
