@@ -1,7 +1,5 @@
 // (beautified)
-console.log(function() {
-    return 1 + .1 + .1;
-}());
+console.log(1 + .1 + .1);
 // output: 1.2000000000000002
 // 
 // minify: 1.2
