@@ -22,7 +22,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
     reduce_options = reduce_options || {};
     var print_options = {};
     [
-        "ie8",
+        "ie",
         "v8",
         "webkit",
     ].forEach(function(name) {
@@ -46,7 +46,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
     if (verbose) {
         log("// Node.js " + process.version + " on " + os.platform() + " " + os.arch());
     }
-    if (differs.error && [ "DefaultsError", "SyntaxError" ].indexOf(differs.error.name) < 0) {
+    if (differs && differs.error && [ "DefaultsError", "SyntaxError" ].indexOf(differs.error.name) < 0) {
         test_for_diff = test_minify;
         differs = test_for_diff(testcase, minify_options, result_cache, max_timeout);
     }
@@ -132,6 +132,8 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                 return;
             }
             if (parent instanceof U.AST_VarDef && parent.name === node) return;
+            // preserve class methods
+            if (parent instanceof U.AST_ClassMethod && parent.value === node) return;
             // preserve exports
             if (parent instanceof U.AST_ExportDeclaration) return;
             if (parent instanceof U.AST_ExportDefault) return;
@@ -147,6 +149,9 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
             if (parent instanceof U.AST_For && parent.init === node && node instanceof U.AST_Definitions) return node;
             // preserve for (xxx in/of ...)
             if (parent instanceof U.AST_ForEnumeration && parent.init === node) return node;
+            // preserve super(...)
+            if (node.TYPE == "Call" && node.expression instanceof U.AST_Super) return;
+            if (node instanceof U.AST_Super && parent.TYPE == "Call" && parent.expression === node) return node;
 
             // node specific permutations with no parent logic
 
@@ -210,9 +215,9 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                     // hoist and return expressions from the IIFE function expression
                     var seq = [];
                     node.expression.body.forEach(function(node) {
-                        var expr = expr instanceof U.AST_Exit ? node.value : node.body;
+                        var expr = node instanceof U.AST_Exit ? node.value : node.body;
                         if (expr instanceof U.AST_Node && !U.is_statement(expr) && can_hoist(expr)) {
-                            // collect expressions from each statements' body
+                            // collect expressions from each statement's body
                             seq.push(expr);
                         }
                     });
@@ -390,7 +395,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
                 var expr = [
                     node.expression,                         // switch expression
                     node.body[0] && node.body[0].expression, // first case expression or undefined
-                    node.body[0] && node.body[0],            // first case body or undefined
+                    node.body[0],                            // first case body or undefined
                 ][ (node.start._permute * steps | 0) % 4 ];
                 node.start._permute += step;
                 if (expr && (!(expr instanceof U.AST_Statement) || !has_loopcontrol(expr, node, parent))) {
@@ -543,7 +548,7 @@ module.exports = function reduce_test(testcase, minify_options, reduce_options)
             }));
             var before_iterations = testcase;
             for (var c = 0; c < max_iterations; ++c) {
-                if (verbose && pass == 1 && c % 25 == 0) {
+                if (verbose && c % (pass == 1 ? 25 : 100) == 0) {
                     log("// reduce test pass " + pass + ", iteration " + c + ": " + testcase.length + " bytes");
                 }
                 var CHANGED = false;
@@ -721,7 +726,7 @@ function to_statement_init(node) {
     return node instanceof U.AST_Const || node instanceof U.AST_Let ? new U.AST_BlockStatement({
         body: [ node ],
         start: {},
-    }) : to_statement(node);;
+    }) : to_statement(node);
 }
 
 function wrap_with_console_log(node) {
