@@ -152,6 +152,7 @@ var SUPPORT = function(matrix) {
     logical_assignment: "[].p ??= 0;",
     new_target: "function f() { new.target; }",
     nullish: "0 ?? 0",
+    optional_chaining: "0?.p",
     rest: "var [...a] = [];",
     rest_object: "var {...a} = {};",
     spread: "[...[]];",
@@ -333,6 +334,7 @@ var VAR_NAMES = [
     "arguments",
     "async",
     "await",
+    "let",
     "yield",
 ];
 var INITIAL_NAMES_LEN = VAR_NAMES.length;
@@ -351,10 +353,11 @@ var TYPEOF_OUTCOMES = [
 ];
 
 var avoid_vars = [];
-var block_vars = [];
+var block_vars = [ "let" ];
 var lambda_vars = [];
 var unique_vars = [];
 var classes = [];
+var allow_this = true;
 var async = false;
 var has_await = false;
 var export_default = false;
@@ -397,10 +400,11 @@ function mayDefer(code) {
 
 function createTopLevelCode() {
     VAR_NAMES.length = INITIAL_NAMES_LEN; // prune any previous names still in the list
-    block_vars.length = 0;
+    block_vars.length = 1;
     lambda_vars.length = 0;
     unique_vars.length = 0;
     classes.length = 0;
+    allow_this = true;
     async = false;
     has_await = false;
     export_default = false;
@@ -445,9 +449,9 @@ function addTrailingComma(list) {
 
 function createParams(was_async, was_generator, noDuplicate) {
     var save_async = async;
-    if (was_async) async = true;
+    if (!async) async = was_async;
     var save_generator = generator;
-    if (was_generator) generator = true;
+    if (!generator) generator = was_generator;
     var len = unique_vars.length;
     var params = [];
     for (var n = rng(4); --n >= 0;) {
@@ -569,9 +573,9 @@ function createAssignmentPairs(recurmax, stmtDepth, canThrow, nameLenBefore, was
     function createName() {
         unique_vars.push("a", "b", "c", "undefined", "NaN", "Infinity");
         var save_async = async;
-        if (was_async) async = true;
+        if (!async) async = was_async;
         var save_generator = generator;
-        if (was_generator) generator = true;
+        if (!generator) generator = was_generator;
         var name = createVarName(MANDATORY);
         generator = save_generator;
         async = save_async;
@@ -982,8 +986,17 @@ function createStatement(recurmax, canThrow, canBreak, canContinue, cannotReturn
         var label = createLabel(canBreak, canContinue);
         canBreak = label.break || enableLoopControl(canBreak, CAN_BREAK);
         canContinue = label.continue || enableLoopControl(canContinue, CAN_CONTINUE);
-        var key = rng(10) ? "key" + loop : getVarName(NO_CONST);
         var of = SUPPORT.for_of && rng(20) == 0;
+        var key;
+        if (rng(10)) {
+            key = "key" + loop;
+        } else if (bug_for_of_async && of) {
+            addAvoidVar("async");
+            key = getVarName(NO_CONST);
+            removeAvoidVar("async");
+        } else {
+            key = getVarName(NO_CONST);
+        }
         var init = "";
         if (!/^key/.test(key)) {
             if (!(of && bug_for_of_var) && rng(10) == 0) init = "var ";
@@ -1179,7 +1192,7 @@ function createStatement(recurmax, canThrow, canBreak, canContinue, cannotReturn
 
 function createSwitchParts(recurmax, n, canThrow, canBreak, canContinue, cannotReturn, stmtDepth) {
     var hadDefault = false;
-    var s = [""];
+    var s = [ "" ];
     canBreak = enableLoopControl(canBreak, CAN_BREAK);
     while (n-- > 0) {
         //hadDefault = n > 0; // disables weird `default` clause positioning (use when handling destabilizes)
@@ -1478,12 +1491,16 @@ function _createExpression(recurmax, noComma, stmtDepth, canThrow) {
         return createValue() + " in " + createObjectLiteral(recurmax, stmtDepth, canThrow);
       case p++:
         var name = getVarName();
-        var s = name + "[" + createExpression(recurmax, COMMA_OK, stmtDepth, canThrow) + "]";
-        return canThrow && rng(20) == 0 ? s : name + " && " + s;
+        var prop = "[" + createExpression(recurmax, COMMA_OK, stmtDepth, canThrow) + "]";
+        if (SUPPORT.optional_chaining && rng(50) == 0) return name + "?." + prop;
+        if (canThrow && rng(20) == 0) return name + prop;
+        return name + " && " + name + prop;
       case p++:
         var name = getVarName();
-        var s = name + "." + getDotKey();
-        return canThrow && rng(20) == 0 ? s : name + " && " + s;
+        var prop = getDotKey();
+        if (SUPPORT.optional_chaining && rng(50) == 0) return name + "?." + prop;
+        if (canThrow && rng(20) == 0) return name + "." + prop;
+        return name + " && " + name + "." + prop;
       case p++:
       case p++:
         var name = getVarName();
@@ -1525,7 +1542,16 @@ function _createExpression(recurmax, noComma, stmtDepth, canThrow) {
             name = rng(3) == 0 ? getVarName() : "f" + rng(funcs + 2);
         } while (name in called && !called[name]);
         called[name] = true;
-        return mayDefer("typeof " + name + ' == "function" && --_calls_ >= 0 && ' + name + createArgs(recurmax, stmtDepth, canThrow));
+        var args = createArgs(recurmax, stmtDepth, canThrow);
+        var call = "typeof " + name + ' == "function" && --_calls_ >= 0 && ' + name + args;
+        if (canThrow) {
+            if (SUPPORT.optional_chaining && args[0] != "`" && rng(50) == 0) {
+                call = "--_calls_ >= 0 && " + name + "?." + args;
+            } else if (rng(20) == 0) {
+                call = "--_calls_ >= 0 && " + name + args;
+            }
+        }
+        return mayDefer(call);
     }
     _createExpression.N = p;
     return _createExpression(recurmax, noComma, stmtDepth, canThrow);
@@ -1610,6 +1636,10 @@ var KEYS = [
     "1.5",
     "3",
 ].concat(SAFE_KEYS);
+SAFE_KEYS = SAFE_KEYS.concat(SAFE_KEYS);
+SAFE_KEYS = SAFE_KEYS.concat(SAFE_KEYS);
+SAFE_KEYS = SAFE_KEYS.concat(SAFE_KEYS);
+SAFE_KEYS.push("__proto__");
 
 function getDotKey(assign) {
     var key;
@@ -1704,6 +1734,8 @@ function createObjectFunction(recurmax, stmtDepth, canThrow, internal, isClazz)
         fn = function(defns) {
             if (generator) name = "*" + name;
             if (async) name = "async "+ name;
+            var save_allow = allow_this;
+            if (internal == "super") allow_this = false;
             s = [
                 name + "(" + createParams(save_async, save_generator, NO_DUPLICATE) + "){",
                 strictMode(),
@@ -1711,6 +1743,7 @@ function createObjectFunction(recurmax, stmtDepth, canThrow, internal, isClazz)
             ];
             s.push(_createStatements(2, recurmax, canThrow, CANNOT_BREAK, CANNOT_CONTINUE, CANNOT_RETURN, stmtDepth));
             if (internal == "super") s.push("super" + createArgs(recurmax, stmtDepth, canThrow, NO_TEMPLATE) + ";");
+            allow_this = save_allow;
             if (/^(constructor|super)$/.test(internal) || rng(10) == 0) for (var i = rng(4); --i >= 0;) {
                 s.push(rng(2) ? createSuperAssignment(recurmax, stmtDepth, canThrow) : createThisAssignment(recurmax, stmtDepth, canThrow));
             }
@@ -1727,8 +1760,9 @@ function createObjectFunction(recurmax, stmtDepth, canThrow, internal, isClazz)
 
 function createObjectLiteral(recurmax, stmtDepth, canThrow) {
     recurmax--;
-    var obj = ["({"];
+    var obj = [ "({" ];
     var offset = SUPPORT.spread_object ? 0 : SUPPORT.computed_key ? 2 : 4;
+    var has_proto = false;
     for (var i = rng(6); --i >= 0;) switch (offset + rng(50 - offset)) {
       case 0:
         obj.push("..." + getVarName() + ",");
@@ -1744,7 +1778,12 @@ function createObjectLiteral(recurmax, stmtDepth, canThrow) {
         obj.push(createObjectFunction(recurmax, stmtDepth, canThrow) + ",");
         break;
       default:
-        obj.push(createObjectKey(recurmax, stmtDepth, canThrow) + ": " + createExpression(recurmax, COMMA_OK, stmtDepth, canThrow) + ",");
+        if (has_proto || rng(200)) {
+            obj.push(createObjectKey(recurmax, stmtDepth, canThrow) + ": " + createExpression(recurmax, COMMA_OK, stmtDepth, canThrow) + ",");
+        } else {
+            obj.push("__proto__: " + createExpression(recurmax, COMMA_OK, stmtDepth, canThrow) + " || {},");
+            has_proto = true;
+        }
         break;
     }
     obj.push("})");
@@ -1787,7 +1826,7 @@ function createClassLiteral(recurmax, stmtDepth, canThrow, name) {
         if (SUPPORT.class_field && rng(2)) {
             s += internal || createObjectKey(recurmax, stmtDepth, canThrow);
             if (rng(5)) {
-                async = false;
+                async = bug_async_class_await && fixed && 0;
                 generator = false;
                 s += " = " + createExpression(recurmax, NO_COMMA, stmtDepth, fixed ? canThrow : CANNOT_THROW);
                 generator = save_generator;
@@ -1941,7 +1980,7 @@ function createValue() {
     var v;
     do {
         v = VALUES[rng(VALUES.length)];
-    } while (v == "new.target" && rng(200));
+    } while (v == "new.target" && rng(200) || !allow_this && v == "this");
     return v;
 }
 
@@ -1989,9 +2028,10 @@ function removeAvoidVar(name) {
 function isBannedKeyword(name) {
     switch (name) {
       case "arguments":
+      case "let":
         return in_class;
       case "await":
-        return async;
+        return async !== false;
       case "yield":
         return generator || in_class;
     }
@@ -2036,11 +2076,12 @@ function createVarName(maybe, dontStore) {
 if (require.main !== module) {
     exports.createTopLevelCode = createTopLevelCode;
     exports.num_iterations = num_iterations;
+    exports.verbose = verbose;
     return;
 }
 
-function run_code(code, toplevel) {
-    return sandbox.run_code(sandbox.patch_module_statements(code), toplevel);
+function run_code(code, toplevel, timeout) {
+    return sandbox.run_code(sandbox.patch_module_statements(code), toplevel, timeout);
 }
 
 function writeln(stream, msg) {
@@ -2098,7 +2139,7 @@ function log_suspects(minify_options, component) {
     var defs = default_options[component];
     var toplevel = sandbox.has_toplevel(minify_options);
     var suspects = Object.keys(defs).filter(function(name) {
-        var flip = name == "keep_fargs";
+        var flip = component == "compress" && name == "keep_fargs";
         if (flip !== (name in options ? options : defs)[name]) {
             var m = JSON.parse(JSON.stringify(minify_options));
             var o = JSON.parse(JSON.stringify(options));
@@ -2217,7 +2258,7 @@ function sort_globals(code) {
             return (typeof global[n] == "function") - (typeof global[m] == "function")
                 || (m < n ? -1 : m > n ? 1 : 0);
         };
-    } + "(this));" + code);
+    } + "(this));\n" + code);
     if (!Array.isArray(globals)) {
         errorln();
         errorln();
@@ -2249,12 +2290,26 @@ function fuzzy_match(original, uglified) {
     return true;
 
     function collect(input, nums) {
-        return input.replace(/-?([1-9][0-9]*(\.[0-9]+)?|0\.[0-9]+)(e-?[1-9][0-9]*)?/ig, function(num) {
+        return input.replace(/-?([1-9][0-9]*(\.[0-9]+)?|0\.[0-9]+)(e-?[1-9][0-9]*)?/gi, function(num) {
             return "<|" + nums.push(+num) + "|>";
         });
     }
 }
 
+function patch_proto() {
+    [ Array, Boolean, Error, Function, Number, Object, RegExp, String ].forEach(function(type) {
+        [ "toString", "valueOf" ].forEach(function(prop) {
+            type.prototype[prop] = function(fn) {
+                return function() {
+                    try {
+                        return fn.apply(this, arguments);
+                    } catch (e) {}
+                };
+            }(type.prototype[prop]);
+        });
+    });
+}
+
 function is_error_timeout(ex) {
     return /timed out/.test(ex.message);
 }
@@ -2264,7 +2319,7 @@ function is_error_in(ex) {
 }
 
 function is_error_spread(ex) {
-    return ex.name == "TypeError" && /Found non-callable @@iterator| is not iterable| is not a function/.test(ex.message);
+    return ex.name == "TypeError" && /Found non-callable @@iterator| is not iterable| not a function/.test(ex.message);
 }
 
 function is_error_recursion(ex) {
@@ -2297,7 +2352,7 @@ function patch_try_catch(orig, toplevel) {
         tries: [],
     } ];
     var tail_throw = '\nif (typeof UFUZZ_ERROR == "object") throw UFUZZ_ERROR;\n';
-    var re = /(?:(?:^|[\s{}):;])try|}\s*catch\s*\(([^)[{]+)\)|}\s*finally)\s*(?={)/g;
+    var re = /(?:(?:^|[\s{}):;])try|}\s*catch\s*\(([^()[{]+)\)|}\s*finally)\s*(?={)/g;
     while (stack.length) {
         var code = stack[0].code;
         var offset = stack[0].offset;
@@ -2383,6 +2438,8 @@ if (SUPPORT.arrow && SUPPORT.async && SUPPORT.rest && typeof sandbox.run_code("a
         return ex.name == "SyntaxError" && ex.message == "Rest parameter must be last formal parameter";
     };
 }
+var bug_async_class_await = SUPPORT.async && SUPPORT.class_field && typeof sandbox.run_code("var await; async function f() { class A { static p = await; } }") != "string";
+var bug_for_of_async = SUPPORT.for_await_of && typeof sandbox.run_code("var async; for (async of []);") != "string";
 var bug_for_of_var = SUPPORT.for_of && SUPPORT.let && typeof sandbox.run_code("try {} catch (e) { for (var e of []); }") != "string";
 if (SUPPORT.destructuring && typeof sandbox.run_code("console.log([ 1 ], {} = 2);") != "string") {
     beautify_options.output.v8 = true;
@@ -2412,8 +2469,10 @@ for (var round = 1; round <= num_iterations; round++) {
         println("original result:");
         println(result);
         println();
-        // ignore v8 parser bug
-        return bug_async_arrow_rest(result);
+            // ignore v8 parser bug
+        return bug_async_arrow_rest(result)
+            // ignore runtime platform bugs
+            || result.message == "Script execution aborted.";
     })) continue;
     minify_options.forEach(function(options) {
         var o = JSON.parse(options);
@@ -2428,19 +2487,27 @@ for (var round = 1; round <= num_iterations; round++) {
             ok = sandbox.same_stdout(original_result, uglify_result);
             // ignore v8 parser bug
             if (!ok && bug_async_arrow_rest(uglify_result)) ok = true;
+            // ignore runtime platform bugs
+            if (!ok && uglify_result.message == "Script execution aborted.") ok = true;
             // handle difference caused by time-outs
-            if (!ok && errored && is_error_timeout(original_result)) {
-                if (is_error_timeout(uglify_result)) {
-                    // ignore difference in error message
-                    ok = true;
-                } else {
+            if (!ok) {
+                if (errored && is_error_timeout(original_result)) {
+                    if (is_error_timeout(uglify_result)) {
+                        // ignore difference in error message
+                        ok = true;
+                    } else {
+                        // ignore spurious time-outs
+                        if (!orig_result[toplevel ? 3 : 2]) orig_result[toplevel ? 3 : 2] = run_code(original_code, toplevel, 10000);
+                        ok = sandbox.same_stdout(orig_result[toplevel ? 3 : 2], uglify_result);
+                    }
+                } else if (is_error_timeout(uglify_result)) {
                     // ignore spurious time-outs
-                    if (!orig_result[toplevel ? 3 : 2]) orig_result[toplevel ? 3 : 2] = run_code(original_code, toplevel, 10000);
-                    ok = sandbox.same_stdout(orig_result[toplevel ? 3 : 2], uglify_result);
+                    var waited_result = run_code(uglify_code, toplevel, 10000);
+                    ok = sandbox.same_stdout(original_result, waited_result);
                 }
             }
             // ignore declaration order of global variables
-            if (!ok && !toplevel) {
+            if (!ok && !toplevel && uglify_result.name != "SyntaxError" && original_result.name != "SyntaxError") {
                 ok = sandbox.same_stdout(run_code(sort_globals(original_code)), run_code(sort_globals(uglify_code)));
             }
             // ignore numerical imprecision caused by `unsafe_math`
@@ -2459,18 +2526,23 @@ for (var round = 1; round <= num_iterations; round++) {
             if (!ok && errored && uglify_result.name == "ReferenceError" && original_result.name == "ReferenceError") ok = true;
             // ignore difference due to implicit strict-mode in `class`
             if (!ok && /\bclass\b/.test(original_code)) {
-                var original_strict = run_code('"use strict";' + original_code, toplevel);
-                var uglify_strict = run_code('"use strict";' + uglify_code, toplevel);
-                if (typeof original_strict != "string") {
-                    ok = typeof uglify_strict != "string";
+                var original_strict = run_code('"use strict";\n' + original_code, toplevel);
+                if (/^(Syntax|Type)Error$/.test(uglify_result.name)) {
+                    ok = typeof original_strict != "string";
                 } else {
-                    ok = sandbox.same_stdout(original_strict, uglify_strict);
+                    ok = sandbox.same_stdout(original_strict, uglify_result);
                 }
             }
             // ignore difference in error message caused by `import` symbol redeclaration
             if (!ok && errored && /\bimport\b/.test(original_code)) {
                 if (is_error_redeclaration(uglify_result) && is_error_redeclaration(original_result)) ok = true;
             }
+            // ignore difference due to `__proto__` assignment
+            if (!ok && /\b__proto__\b/.test(original_code)) {
+                var original_proto = run_code("(" + patch_proto + ")();\n" + original_code, toplevel);
+                var uglify_proto = run_code("(" + patch_proto + ")();\n" + uglify_code, toplevel);
+                ok = sandbox.same_stdout(original_proto, uglify_proto);
+            }
             // ignore difference in error message caused by `in`
             if (!ok && errored && is_error_in(uglify_result) && is_error_in(original_result)) ok = true;
             // ignore difference in error message caused by spread syntax
